#====
# Enemy Boss Class
#====

import os, sys, math, random
import pygame
from pygame.locals import *
from equates import *
from helpers import *
from weapon_task import *
from weapon_magicwand import *
from weapon_creditcard import *

class EnemyBossSprite(pygame.sprite.Sprite):

	def __init__(self,position):	
		pygame.sprite.Sprite.__init__(self)				
		self.image, self.rect = load_image('enemy_boss.png',None,position)
		if self.rect == None:
			self.rect = self.image.get_rect()
		self.src_image = self.image
		self.max_health = BOSS_HEALTH_MAX
		self.current_health = self.max_health
		self.speed = BOSS_START_SPEED
		self.position = position
		self.direction = random.randrange(0,360)
		self.alert_radius = 150
		self.previous_direction_increment = 0
		self.fire_timer = 0
		self.weapon_task_sound = load_sound('weapon_task.ogg')
		self.weapon_magicwand_sound = load_sound('weapon_magicwand.ogg')
		self.weapon_creditcard_sound = load_sound('weapon_creditcard.ogg')
		self.birth_sound = load_sound('enemy_boss_birth.ogg')
		#self.birth_sound.play()
		self.death_sound = load_sound('enemy_boss_death.ogg')
		self.random_call_timer = 0
		"""
		self.random_call_1_sound = load_sound('enemy_boss_random_1.ogg')
		self.random_call_2_sound = load_sound('enemy_boss_random_2.ogg')
		self.random_call_3_sound = load_sound('enemy_boss_random_3.ogg')
		self.random_call_4_sound = load_sound('enemy_boss_random_4.ogg')
		self.random_call_5_sound = load_sound('enemy_boss_random_5.ogg')
		self.random_call_6_sound = load_sound('enemy_boss_random_6.ogg')
		"""
		
	def update(self,deltat,weapon_task_group,weapon_magicwand_group,weapon_creditcard_group):
		# .. Init
		
		# .. Fire Weapon?
		self.fire_timer += deltat
		if self.fire_timer >= BOSS_WEAPON_FIRE_TIMER:
			self.fire_timer = 0
			# .. Which Weapon?
			choice = random.randrange(1,3)
			# .. Task
			if choice == 1:
				if random.randrange(1,100) > 50:
					weapon_task_group.add(WeaponTaskSprite(self.position,self.direction-90,WEAPON_LIFESPAN_MEDIUM))
					weapon_task_group.add(WeaponTaskSprite(self.position,self.direction+90,WEAPON_LIFESPAN_MEDIUM))
					self.weapon_task_sound.play()
			# .. Magic Wand
			elif choice == 2:
				if random.randrange(1,100) > 50:
					weapon_magicwand_group.add(WeaponMagicWandSprite(self.position,self.direction-90,WEAPON_LIFESPAN_MEDIUM))
					weapon_magicwand_group.add(WeaponMagicWandSprite(self.position,self.direction+90,WEAPON_LIFESPAN_MEDIUM))
					self.weapon_magicwand_sound.play()
			# .. Credit Card
			elif choice == 2:
				if random.randrange(1,100) > 50:
					weapon_creditcard_group.add(WeaponCreditCardSprite(self.position,self.direction-90,WEAPON_LIFESPAN_MEDIUM))
					weapon_creditcard_group.add(WeaponCreditCardSprite(self.position,self.direction+90,WEAPON_LIFESPAN_MEDIUM))
					self.weapon_creditcard_sound.play()
		"""
		# .. Random Call?
		self.random_call_timer += deltat
		if self.random_call_timer > BOSS_RANDOM_CALL_TIMER:
			if random.randrange(1,100) > 85:
				choice = random.randrange(1,6)
				if choice == 1:
					if random.randrange(1,100) > 50:
						self.random_call_1_sound.play()
				elif choice == 2:
					if random.randrange(1,100) > 50:
						self.random_call_2_sound.play()
				elif choice == 3:
					if random.randrange(1,100) > 50:
						self.random_call_3_sound.play()
				elif choice == 4:
					if random.randrange(1,100) > 50:
						self.random_call_4_sound.play()
				elif choice == 5:
					if random.randrange(1,100) > 50:
						self.random_call_5_sound.play()
				elif choice == 6:
					if random.randrange(1,100) > 50:
						self.random_call_6_sound.play()
		"""				
		# .. Random Direction
		if random.randrange(1,20) > 15:
			if (self.previous_direction_increment >= 0) and (random.randrange(1,10) > 3):
				self.direction += BOSS_TURN_SPEED
				self.previous_direction_increment = BOSS_TURN_SPEED
			else:
				self.direction -= BOSS_TURN_SPEED
				self.previous_direction_increment = -BOSS_TURN_SPEED
		# .. Move
		x,y = self.position
		rad = self.direction * math.pi / 180
		x += -self.speed*math.sin(rad)
		y += -self.speed*math.cos(rad)
		# .. boundary check
		if x < 0: x = MAX_XPOS
		elif x > MAX_XPOS: x = 0
		if y < 0: y = MAX_YPOS
		elif y > MAX_YPOS: y = 0
		# .. continue
		self.position = (x,y)
		self.image = pygame.transform.rotate(self.src_image,self.direction)
		self.rect = self.image.get_rect()
		self.rect.center = self.position
		
		# .. Update Tail, assume that pieces are in order
		#tail_group.update(deltat,self.direction)
		
	def draw_health(self,display):
		font = pygame.font.Font(DEFAULT_FONT,8)
		health_percentage = int(float(self.current_health) / float(self.max_health) * 100)
		text = font.render(str(health_percentage)+"%",1,(255,255,255),(0,0,0))
		textpos = self.rect.bottomleft
		textpos = (textpos[0],textpos[1]+5)
		display.blit(text,textpos)