#====
# Enemy CEO Class
#====

import os, sys, math, random
import pygame
from pygame.locals import *
from equates import *
from helpers import *
from weapon_magicwand import *

class EnemyCEOSprite(pygame.sprite.Sprite):

	def __init__(self,position):	
		pygame.sprite.Sprite.__init__(self)				
		self.image, self.rect = load_image('enemy_ceo.png',None,position)
		if self.rect == None:
			self.rect = self.image.get_rect()
		self.src_image = self.image
		self.max_health = CEO_HEALTH_MAX
		self.current_health = self.max_health
		self.speed = CEO_START_SPEED
		self.position = position
		self.direction = random.randrange(0,360)
		self.alert_radius = 150
		self.previous_direction_increment = 0
		self.fire_timer = 0
		self.weapon_sound = load_sound('weapon_magicwand.ogg')
		self.death_sound = load_sound('enemy_ceo_death.ogg')
		
	def update(self,deltat,player_position,weapon_magicwand_group):
		# .. Fire Weapon?
		self.fire_timer += deltat
		if self.fire_timer >= CEO_FIRE_TIMER:
			self.fire_timer = 0
			if random.randrange(1,100) > 50:
				weapon_magicwand_group.add(WeaponMagicWandSprite(self.position,self.direction-90,WEAPON_LIFESPAN_MEDIUM))
				weapon_magicwand_group.add(WeaponMagicWandSprite(self.position,self.direction+90,WEAPON_LIFESPAN_MEDIUM))
				self.weapon_sound.play()
		# .. Random Direction
		if random.randrange(1,20) > 15:
			if (self.previous_direction_increment >= 0) and (random.randrange(1,10) > 3):
				self.direction += CEO_TURN_SPEED
				self.previous_direction_increment = CEO_TURN_SPEED
			else:
				self.direction -= CEO_TURN_SPEED
				self.previous_direction_increment = -CEO_TURN_SPEED
		# .. Move
		x,y = self.position
		rad = self.direction * math.pi / 180
		x += -self.speed*math.sin(rad)
		y += -self.speed*math.cos(rad)
		# .. boundary check
		if x < 0: x = MAX_XPOS
		elif x > MAX_XPOS: x = 0
		if y < 0: y = MAX_YPOS
		elif y > MAX_YPOS: y = 0
		# .. continue
		self.position = (x,y)
		self.image = pygame.transform.rotate(self.src_image,self.direction)
		self.rect = self.image.get_rect()
		self.rect.center = self.position
		
		# .. Update Tail, assume that pieces are in order
		#tail_group.update(deltat,self.direction)
		
	def draw_health(self,display):
		font = pygame.font.Font(DEFAULT_FONT,8)
		health_percentage = int(float(self.current_health) / float(self.max_health) * 100)
		text = font.render(str(health_percentage)+"%",1,(255,255,255),(0,0,0))
		textpos = self.rect.bottomleft
		textpos = (textpos[0],textpos[1]+5)
		display.blit(text,textpos)