#====
# Enemy Sycophant Class
#====

import os, sys, math, random
import pygame
from pygame.locals import *
from equates import *
from helpers import *

class EnemySycophantChildSprite(pygame.sprite.Sprite):
	def __init__(self,position,direction,lifespan):
		pygame.sprite.Sprite.__init__(self)
		self.image, self.rect = load_image('enemy_sycophant_child.png',None,position)
		if self.rect == None:
			self.rect = rect
		self.position = position
		self.direction = direction
		self.speed = SYCOPHANT_CHILD_SPEED
		self.lifespan = lifespan
		self.age = 0

	def update(self,deltat):	
		# .. Life Check
		self.age += 1
		if self.age == self.lifespan:
			self.kill()
		# .. Calc New Position
		x,y = self.position
		rad = self.direction * math.pi / 180
		x += -self.speed*math.sin(rad)
		y += -self.speed*math.cos(rad)
		# .. boundary check
		if x < 0: x = MAX_XPOS
		elif x > MAX_XPOS: x = 0
		if y < 0: y = MAX_YPOS
		elif y > MAX_YPOS: y = 0
		# .. continue
		self.position = (x,y)		
		self.rect = self.image.get_rect()
		self.rect.center = self.position

class EnemySycophantSprite(pygame.sprite.Sprite):

	def __init__(self,position):	
		pygame.sprite.Sprite.__init__(self)				
		self.image, self.rect = load_image('enemy_sycophant.png',None,position)
		if self.rect == None:
			self.rect = self.image.get_rect()
		self.src_image = self.image
		self.max_health = SYCOPHANT_HEALTH_MAX
		self.current_health = self.max_health
		self.speed = SYCOPHANT_START_SPEED
		self.position = position
		self.direction = random.randrange(0,360)
		self.alert_radius = 150
		self.previous_direction_increment = 0
		self.fire_timer = 0
		self.death_sound = load_sound('enemy_sycophant_death.ogg')
		self.lifespan = SYCOPHANT_LIFESPAN
		self.age = 0		
		self.timer_sound = load_sound('weapon_timer.ogg')
		self.timer_sound_played = 0
		self.detonate_sound = load_sound('weapon_detonate.ogg')
		
	def update(self,deltat,player_position,sycophant_child_group):
		# .. Life Check
		self.age += 1	
		# .. Detonate?
		if self.age == self.lifespan:
			self.detonate(sycophant_child_group)
			self.kill()
		elif self.age >= self.lifespan-20 and self.timer_sound_played == 0:
			self.timer_sound_played = 1
			self.timer_sound.play()
		# .. Random Direction
		if random.randrange(1,20) > 15:
			if (self.previous_direction_increment >= 0) and (random.randrange(1,10) > 3):
				self.direction += SYCOPHANT_TURN_SPEED
				self.previous_direction_increment = SYCOPHANT_TURN_SPEED
			else:
				self.direction -= SYCOPHANT_TURN_SPEED
				self.previous_direction_increment = -SYCOPHANT_TURN_SPEED
		# .. Move
		x,y = self.position
		rad = self.direction * math.pi / 180
		x += -self.speed*math.sin(rad)
		y += -self.speed*math.cos(rad)
		# .. boundary check
		if x < 0: x = MAX_XPOS
		elif x > MAX_XPOS: x = 0
		if y < 0: y = MAX_YPOS
		elif y > MAX_YPOS: y = 0
		# .. continue
		self.position = (x,y)
		self.image = pygame.transform.rotate(self.src_image,self.direction)
		self.rect = self.image.get_rect()
		self.rect.center = self.position
		
	def draw_health(self,display):
		font = pygame.font.Font(DEFAULT_FONT,8)
		health_percentage = int(float(self.current_health) / float(self.max_health) * 100)
		text = font.render(str(health_percentage)+"%",1,(255,255,255),(0,0,0))
		textpos = self.rect.bottomleft
		textpos = (textpos[0],textpos[1]+5)
		display.blit(text,textpos)
		
	def detonate(self,sycophant_child_group):
		self.detonate_sound.play()
		pos = 0
		while pos <= 360:
			sycophant_child_group.add(EnemySycophantChildSprite(self.position,pos,WEAPON_LIFESPAN_SHORT))
			pos += 45