# INITALISATION
import os, sys, math, random
import pygame
from pygame.locals import *
from equates import *
from helpers import *
from player import *
from enemy_manager import *
from enemy_visitingsuits import *
from enemy_uppermanagement import *
from enemy_ceo import *
from enemy_boardmember import *
from enemy_sycophant import *
from enemy_boss import *
from weapon_one import *
from weapon_two import *
from weapon_three import *
from weapon_three_base import *
from weapon_tag import *
from weapon_task import *
from weapon_magicwand import *
from weapon_creditcard import *
from powerup_lifespan import *
from powerup_damage import *
from powerup_multi import *
from powerup_through import *
from powerup_health import *
from powerup_weapon_two import *
from powerup_maxhealth import *
from vec2d import *

if not pygame.font: print 'Warning, fonts disabled'
if not pygame.mixer: print 'Warning, sound disabled'

#====
# MAIN CLASS .. Handles Init and Creation of the Game
#====
class GameMain:
	
	def __init__(self,width=1024,height=768):
		pygame.init()
		# .. set window
		self.width = width
		self.height = height
		self.set_icon()
		self.screen = pygame.display.set_mode((self.width,self.height))
		self.clock = pygame.time.Clock()
		self.gamestate = STATE_INIT

		self.playerbase_rect = pygame.Rect(PLAYER_BASE_LEFT,PLAYER_BASE_TOP,PLAYER_BASE_WIDTH,PLAYER_BASE_HEIGHT)
		self.managers_alive = self.visitingsuits_alive = self.uppermanagement_alive = self.ceos_alive = self.boardmembers_alive = self.sycophants_alive = self.enemies_alive = 0
		
		self.show_menu = self.read_menu = self.show_instructions = self.show_end = 0
		
		self.boss_initialised = 0
	
	def MainLoop(self):
		# Bring it, this is where it all goes down
		# .. well, most of it
		
		# .. fire up the engines
		self.init_player()
		self.init_managers()
		self.init_visitingsuits()
		self.init_uppermanagement()
		self.init_ceos()
		self.init_boardmembers()
		self.init_sycophants()
		self.init_sycophant_children()
		self.init_boss()
		self.init_weapon_one()
		self.init_weapon_two()
		self.init_weapon_two_shrapnel()
		self.init_weapon_three_base()
		self.init_weapon_three()
		self.init_weapon_tags()
		self.init_weapon_task()
		self.init_weapon_magicwand()
		self.init_weapon_creditcard()
		self.init_powerup_lifespan()
		self.init_powerup_damage()
		self.init_powerup_multi()
		self.init_powerup_through()
		self.init_powerup_health()
		self.init_powerup_weapon_two()
		self.init_powerup_maxhealth()
		
		# .. if there's a background, do it here
		self.background = pygame.Surface(self.screen.get_size())
		self.background = self.background.convert()
		self.background.fill((0,0,0))
			
		self.gamestate = STATE_BEGIN
		self.show_instructions = 1
		
		# GAME LOOP
		# .. okay, despite the above, THIS is where it's at
		while 1:
			self.deltat = self.clock.tick(30)		

			#/////////////
			# GAME STATES
			# - Okay, think of these as the little "Shells" in which it is possible to 
			# 	do stuff. 
			# - Actually, the "States", whether they be Game, or Player, or sub-states all 
			#		use the same basic methodology. In this game, an Entity can only (at this stage)
			#		have one State at a time.
			#/////////////		
			
			#//////////
			# gamestate == BEGIN
			#//////////
			if self.gamestate == STATE_BEGIN:			
				if self.show_instructions == 1:
					for event in pygame.event.get():
						if not hasattr(event,'key'): continue
						self.down = event.type == KEYDOWN
						if event.type == KEYDOWN:
							self.gamestate = STATE_PLAY
							self.show_instructions = 0
					# RENDERING
					self.render_logic()
			#//////////
			# gamestate == PLAY
			#//////////
			elif self.gamestate == STATE_PLAY:	
				#-----
				# INSTRUCTIONS
				#-----
				if self.show_instructions == 1:
					for event in pygame.event.get():
						if not hasattr(event,'key'): continue
						self.down = event.type == KEYDOWN
						if event.type == KEYDOWN:
							self.show_instructions = 0
					# RENDERING
					self.render_logic()
				#-----
				# SHOW MENU
				#-----				
				elif self.show_menu == 1:
					# USER INPUT
					for event in pygame.event.get():
						if not hasattr(event,'key'): continue
						self.down = event.type == KEYDOWN
						if event.type == KEYDOWN:
							# .. Continue
							if event.key == K_c:
								self.show_menu = 0
								if self.player.state == PLAYERSTATE_DEAD:
									self.player.state = PLAYERSTATE_ALIVE
									rect = self.screen.get_rect()
									self.player.rebirth(rect.center)
									self.clear_player_base()
							# .. Exit
							elif event.key == K_ESCAPE:
								self.gamestate = STATE_END
							# .. New Game
							elif event.key == K_n: 
								self.new_game()
							# .. Instructions							
							elif event.key == K_i:
								self.show_instructions = 1
								self.show_menu = 0
							
					# RENDERING
					self.render_logic()
				#-----
				# Game is running, because Menu isn't
				#-----
				else:		
					#-------
					# Player ALIVE
					#-------
					if self.player.state == PLAYERSTATE_ALIVE:
						# USER INPUT					
						for event in pygame.event.get():
							if not hasattr(event,'key'): continue
							self.down = event.type == KEYDOWN	
							if event.key == K_RIGHT: self.player.k_right = self.down * -5
							elif event.key == K_LEFT: self.player.k_left = self.down * 5
							elif event.key == K_UP: self.player.k_up = self.down * 2
							elif event.key == K_DOWN: self.player.k_down = self.down * -2
							# .. Show Menu
							elif event.key == K_ESCAPE: 
								self.show_menu = 1
							# .. Stop Movement
							elif event.key == K_SPACE and event.type == KEYDOWN:
								self.player.speed = 0
							# .. Fire Weapon
							elif event.key == K_LCTRL:
								if event.type == KEYDOWN:
									if self.player.current_weapon == WEAPON_ONE:
										# .. Fire Weapon
										self.player.fire_weapon(WEAPON_ONE,self.weapon_one_group)
							# .. Drop Bomb
							elif event.key == K_z:
								if event.type == KEYDOWN:
									self.player.fire_weapon(WEAPON_TWO,self.weapon_two_group)
							# .. Instructions
							elif event.key == K_i:
								self.show_instructions = 1
								
						# WEAPONS/MANAGERS .. Collision Detection .. want this to happen before the 
						self.weapon_one_collide()
						self.weapon_two_collide()
						self.weapon_three_collide()
						#
						# MANAGER .. Collision Detection
						# .. DID THE PLAYER COLLDE WITH A MANAGER?
						#			- because at the moment that's an insta-death
						#
						self.player_collide(self.deltat)
						
					#-------
					# Player DEAD
					#-------
					elif self.player.state == PLAYERSTATE_DEAD:
						self.player.clear_powerups()
						self.show_menu = 1
								
					#/////
					# .. the following logic happens in both Player DEAD/ALIVE states
					#/////
					
					#//////////
					# RENDERING .. rather important
					#//////////
					self.render_logic()
			
			#
			# gamestate == FINISHED
			#
			elif self.gamestate == STATE_FINISHED:
				pygame.time.wait(2000)
				pygame.event.clear()
				self.gamestate = STATE_EPILOGUE
			#
			# gamestate == EPILOGUE
			#
			elif self.gamestate == STATE_EPILOGUE:
				self.screen.fill((0,0,0))
				# .. go on with rendering						
				# .. write text to the screen
				font = pygame.font.Font(DEFAULT_FONT,28)
				text = font.render("Congratulations! You have beaten the Boss.",1,(255,255,255))
				textpos = ((MAX_XPOS/2-300,MAX_YPOS/2-50))
				self.screen.blit(text,textpos)			
				font = pygame.font.Font(DEFAULT_FONT,28)
				text = font.render("But like at work, there are always more ...",1,(255,255,255))
				textpos = ((MAX_XPOS/2-300,MAX_YPOS/2))
				self.screen.blit(text,textpos)			

				pygame.display.flip()					

				#pygame.time.wait(15000)
				
				# Check for key-press
				for event in pygame.event.get():
					if not hasattr(event,'key'): continue
					self.down = event.type == KEYDOWN	
					if event.type == KEYDOWN:
						self.gamestate = STATE_END
						break
					
			#
			# gamestate == END
			#				
			elif self.gamestate == STATE_END:
				sys.exit(0)
	
	#/////////	
	# function
	# -- INIT ENTITIES
	#		- For the various Entities (Player, Managers, Weapons, ..) does the creation stuff
	#/////////
	def init_player(self):
		rect = self.screen.get_rect()
		self.player = PlayerSprite('player.png',rect.center)
		self.player_group = pygame.sprite.RenderPlain((self.player))
	
	def init_managers(self):
		# .. Managers
		self.manager_group = pygame.sprite.RenderUpdates()
		self.add_manager()
		self.add_manager()
	
	def init_visitingsuits(self):
		self.visitingsuits_group = pygame.sprite.RenderUpdates()
		
	def init_uppermanagement(self):
		self.uppermanagement_group = pygame.sprite.RenderUpdates()
		
	def init_ceos(self):
		self.ceo_group = pygame.sprite.RenderUpdates()
		
	def init_boardmembers(self):
		self.boardmember_group = pygame.sprite.RenderUpdates()
		
	def init_sycophants(self):
		self.sycophant_group = pygame.sprite.RenderUpdates()
		
	def init_sycophant_children(self):
		self.sycophant_child_group = pygame.sprite.RenderUpdates()
		
	def init_boss(self):
		self.boss_group = pygame.sprite.RenderUpdates()
	
	def init_weapon_one(self):
		self.weapon_one_group = pygame.sprite.RenderUpdates()
				
	def init_weapon_two(self):
		self.weapon_two_group = pygame.sprite.RenderUpdates()
	
	def init_weapon_two_shrapnel(self):
		self.weapon_two_shrapnel_group = pygame.sprite.RenderUpdates()
		
	def init_weapon_three_base(self):
		self.weapon_three_base_group = pygame.sprite.RenderUpdates()
		
	def init_weapon_three(self):
		self.weapon_three_group = pygame.sprite.RenderUpdates()
		
	def init_weapon_task(self):
		self.weapon_task_group = pygame.sprite.RenderUpdates()
		
	def init_weapon_magicwand(self):
		self.weapon_magicwand_group = pygame.sprite.RenderUpdates()
		
	def init_weapon_creditcard(self):
		self.weapon_creditcard_group = pygame.sprite.RenderUpdates()

	def init_powerup_lifespan(self):
		self.powerup_lifespan_group = pygame.sprite.RenderUpdates()
		
	def init_powerup_damage(self):	
		self.powerup_damage_group = pygame.sprite.RenderUpdates()

	def init_powerup_multi(self):
		self.powerup_multi_group = pygame.sprite.RenderUpdates()
		
	def init_powerup_through(self):
		self.powerup_through_group = pygame.sprite.RenderUpdates()
		
	def init_powerup_health(self):
		self.powerup_health_group = pygame.sprite.RenderUpdates()
		
	def init_powerup_weapon_two(self):
		self.powerup_weapon_two_group = pygame.sprite.RenderUpdates()
		
	def init_powerup_maxhealth(self):
		self.powerup_maxhealth_group = pygame.sprite.RenderUpdates()
		
	def init_weapon_tags(self):
		self.weapon_tag_group = pygame.sprite.RenderUpdates()
		weapon_one_tag_on = WeaponTagSprite('weapon_one_tag.png',(20,WINDOW_HEIGHT-20),0,WEAPON_ONE,1)
		self.weapon_tag_group.add(weapon_one_tag_on)

	def repopulate_weapon_tags(self):
		# .. Remove all Tags
		for s in self.weapon_tag_group:
			s.kill()
		weapon_one_tag = WeaponTagSprite('weapon_one_tag.png',(20,WINDOW_HEIGHT-20),0,WEAPON_ONE,1)
		self.weapon_tag_group.add(weapon_one_tag)
		if self.player.level >= WEAPON_TWO_LEVEL:
			weapon_two_tag = WeaponTagSprite('weapon_two_tag.png',(40,WINDOW_HEIGHT-20),2,WEAPON_TWO,1)
			self.weapon_tag_group.add(weapon_two_tag)
		if self.player.level >= WEAPON_THREE_LEVEL:
			weapon_three_tag = WeaponTagSprite('weapon_three_tag.png',(60,WINDOW_HEIGHT-20),2,WEAPON_THREE,1)
			self.weapon_tag_group.add(weapon_three_tag)			
		
	#/////////	
	# function
	# -- RENDER_LOGIC()
	#		- All Entities and wotnot, to the Screen
	#/////////		
	def render_logic(self):
	
				self.screen.fill((0,0,0))
				# .. go on with rendering		

				# GAME STATE
				if self.gamestate == STATE_PLAY:
					# PLAYER ALIVE
					# - then you "update"
					if self.player.state == PLAYERSTATE_ALIVE:
						if self.show_menu == 0 and self.show_instructions == 0:			# need a better logic method than just checking to see if the menu is showing
							# .. UPDATE the entities on screen
							self.manager_group.update(self.deltat,self.player.position)			# not sure if this should go before or after the Player is updated (ie, it's position)
							self.visitingsuits_group.update(self.deltat,self.player.position)
							self.uppermanagement_group.update(self.deltat,self.player.position,self.weapon_task_group)
							self.ceo_group.update(self.deltat,self.player.position,self.weapon_magicwand_group)
							self.boardmember_group.update(self.deltat,self.player.position,self.weapon_creditcard_group)
							self.sycophant_group.update(self.deltat,self.player.position,self.sycophant_child_group)
							self.sycophant_child_group.update(self.deltat)
							self.boss_group.update(self.deltat,self.weapon_task_group,self.weapon_magicwand_group,self.weapon_creditcard_group)
							self.powerup_lifespan_group.update(self.deltat)
							self.powerup_damage_group.update(self.deltat)
							self.powerup_multi_group.update(self.deltat)
							self.powerup_through_group.update(self.deltat)
							self.powerup_health_group.update(self.deltat)
							self.powerup_weapon_two_group.update(self.deltat)
							self.powerup_maxhealth_group.update(self.deltat)
							# .. Player
							self.player_group.update(self.deltat,self.weapon_three_base_group,self.weapon_three_group)
							player_pos = vec2d(self.player.position[0],self.player.position[1])		# really need to go over and change everything to the Vec2d Class
							enemy_pos, enemy_direction, enemy_speed = self.get_closest_enemy(player_pos)
							self.weapon_one_group.update(self.deltat)
							self.weapon_two_group.update(self.deltat,self.weapon_two_shrapnel_group)
							self.weapon_two_shrapnel_group.update(self.deltat)
							self.weapon_three_group.update(self.deltat)		# weapon_three_base_group is taken care of inside the Player
							self.weapon_task_group.update(self.deltat)
							self.weapon_magicwand_group.update(self.deltat)
							self.weapon_creditcard_group.update(self.deltat)
							
					# .. DRAW the entities on screen .. Order is important: Enemies, Player, Weapons
					self.manager_group.draw(self.screen)
					for s in self.manager_group:			# really should be doing this in the actual Manager Class (and the other Enemy classes), but can't get the "draw" working
						s.draw_health(self.screen)
					self.visitingsuits_group.draw(self.screen)
					for s in self.visitingsuits_group:
						s.draw_health(self.screen)
					self.uppermanagement_group.draw(self.screen)
					for s in self.uppermanagement_group:
						s.draw_health(self.screen)
					self.ceo_group.draw(self.screen)
					for s in self.ceo_group:
						s.draw_health(self.screen)
					self.boardmember_group.draw(self.screen)
					for s in self.boardmember_group:
						s.draw_health(self.screen)
					self.sycophant_group.draw(self.screen)
					for s in self.sycophant_group:
						s.draw_health(self.screen)
					self.sycophant_child_group.draw(self.screen)
					self.boss_group.draw(self.screen)
					for s in self.boss_group:
						s.draw_health(self.screen)
					self.powerup_lifespan_group.draw(self.screen)
					self.powerup_damage_group.draw(self.screen)
					self.powerup_multi_group.draw(self.screen)
					self.powerup_through_group.draw(self.screen)
					self.powerup_health_group.draw(self.screen)
					self.powerup_weapon_two_group.draw(self.screen)
					self.powerup_maxhealth_group.draw(self.screen)
					self.player_group.draw(self.screen)
					self.weapon_one_group.draw(self.screen)
					self.weapon_two_group.draw(self.screen)
					self.weapon_two_shrapnel_group.draw(self.screen)
					self.weapon_three_group.draw(self.screen)
					self.weapon_three_base_group.draw(self.screen)
					self.weapon_task_group.draw(self.screen)
					self.weapon_magicwand_group.draw(self.screen)
					self.weapon_creditcard_group.draw(self.screen)
					
					# GUI				
					# .. Weapon Tags
					self.repopulate_weapon_tags()
					self.weapon_tag_group.draw(self.screen)
				
				# .. write text to the screen
				self.write_text()				
				pygame.display.flip()			
			
	#/////////	
	# function
	# -- WRITE_TEXT()
	#		- Writes the Experience and Counters to the screen
	#/////////
	def write_text(self):
		if pygame.font:
			# .. go through the different pieces of text that should be written on the screen
			if self.gamestate == STATE_PLAY:
				# .. Player Details
				textpos = ((20,10))
				next_xpos = self.player.draw_player_details(self.screen,textpos)
				# .. Totals
				self.draw_totals((next_xpos+50,10))
				# .. Weapon Details
				self.player.draw_weapon_details(self.screen,self.player.current_weapon,(MAX_XPOS-120,10))

			# .. MENU
			if self.show_menu == 1:
				# .. died?
				if self.player.state == PLAYERSTATE_DEAD:					
					font = pygame.font.Font(DEFAULT_FONT,32)
					text_str = "Oh no! You have lost the Space Office Fight!"
					text = font.render(text_str,1,(255,255,0))
					textpos = ((MAX_XPOS/2-font.size(text_str)[0]/2,MAX_YPOS/2-150))
					self.screen.blit(text,textpos)
				# .. other options	
				font = pygame.font.Font(DEFAULT_FONT,28)
				text = font.render("C - Continue",1,(255,255,255))
				textpos = ((MAX_XPOS/2-100,MAX_YPOS/2-50))
				self.screen.blit(text,textpos)
				text = font.render("N - New Game",1,(255,255,255))
				textpos = ((MAX_XPOS/2-100,MAX_YPOS/2))
				self.screen.blit(text,textpos)
				text = font.render("I - Instructions",1,(255,255,255))
				textpos = ((MAX_XPOS/2-100,MAX_YPOS/2+50))
				self.screen.blit(text,textpos)				
				text = font.render("ESC - Exit",1,(255,255,255))
				textpos = ((MAX_XPOS/2-100,MAX_YPOS/2+100))
				self.screen.blit(text,textpos)
	
			# .. INSTRUCTIONS
			if self.show_instructions == 1:	
				# .. Game Name
				font = pygame.font.Font(DEFAULT_FONT,48)
				text = font.render("Space Office Fight",1,(255,52,25))
				textpos = ((20,20))
				self.screen.blit(text,textpos)			
				# .. Heading
				left_xpos = MAX_XPOS/2 - 200
				font = pygame.font.Font(DEFAULT_FONT,36)
				text = font.render("Instructions",1,(255,255,255))
				textpos = ((left_xpos,MAX_YPOS/2-200))
				self.screen.blit(text,textpos)
				# .. Text
				
				font = pygame.font.Font(DEFAULT_FONT,24)
				text_str = "Movement ="
				text = font.render(text_str,1,(255,255,255))
				textpos = ((left_xpos,MAX_YPOS/2-100))
				self.screen.blit(text,textpos)
				next_xpos = textpos[0] + font.size(text_str)[0] + 10
				text = font.render("Arrow Keys",1,(100,25,255))
				textpos = ((next_xpos,MAX_YPOS/2-100))
				self.screen.blit(text,textpos)

				text_str = "Fire ="
				text = font.render(text_str,1,(255,255,255))
				textpos = ((left_xpos,MAX_YPOS/2-50))				
				self.screen.blit(text,textpos)
				next_xpos = textpos[0] + font.size(text_str)[0] + 10
				text = font.render("Left <CTRL>",1,(100,25,255))
				textpos = ((next_xpos,MAX_YPOS/2-50))
				self.screen.blit(text,textpos)
				
				text_str = "Drop Bomb ="
				text = font.render(text_str,1,(255,255,255))
				textpos = ((left_xpos,MAX_YPOS/2))				
				self.screen.blit(text,textpos)
				next_xpos = textpos[0] + font.size(text_str)[0] + 10
				text = font.render("Z",1,(100,25,255))
				textpos = ((next_xpos,MAX_YPOS/2))
				self.screen.blit(text,textpos)
				
				text_str = "Menu ="
				text = font.render(text_str,1,(255,255,255))
				textpos = ((left_xpos,MAX_YPOS/2+50))				
				self.screen.blit(text,textpos)
				next_xpos = textpos[0] + font.size(text_str)[0] + 10
				text = font.render("<ESC>",1,(100,25,255))
				textpos = ((next_xpos,MAX_YPOS/2+50))
				self.screen.blit(text,textpos)
				
				# .. Continue
				font = pygame.font.Font(DEFAULT_FONT,14)
				text = font.render("Press any key to continue ..",1,(255,255,255))
				textpos = ((left_xpos,MAX_YPOS/2+150))
				self.screen.blit(text,textpos)

	def draw_totals(self,next_position):
		# .. Heading
		font = pygame.font.Font(DEFAULT_FONT,12)
		text_str = "Stats"
		text = font.render(text_str,1,(255,255,255))
		textpos = next_position
		self.screen.blit(text,textpos)
		next_position = (next_position[0],next_position[1]+20)	
		# .. Manager Total
		font = pygame.font.Font(DEFAULT_FONT,12)
		text_str = "Managers: "+str(self.player.get_total_managers_killed())
		text = font.render(text_str,1,(192,192,192))
		textpos = next_position
		self.screen.blit(text,textpos)
		next_position = (next_position[0],next_position[1]+20)
		# .. Visiting Suits Total
		font = pygame.font.Font(DEFAULT_FONT,12)
		text_str = "Visiting Suits: "+str(self.player.get_total_visitingsuits_killed())
		text = font.render(text_str,1,(192,192,192))
		textpos = next_position
		self.screen.blit(text,textpos)
		next_position = (next_position[0],next_position[1]+20)
		# .. Upper Management Total
		font = pygame.font.Font(DEFAULT_FONT,12)
		text_str = "Upper Management: "+str(self.player.get_total_uppermanagement_killed())
		text = font.render(text_str,1,(192,192,192))
		textpos = next_position
		self.screen.blit(text,textpos)
		next_position = (next_position[0],next_position[1]+20)
		# .. CEO Total
		font = pygame.font.Font(DEFAULT_FONT,12)
		text_str = "CEOs: "+str(self.player.get_total_ceos_killed())
		text = font.render(text_str,1,(192,192,192))
		textpos = next_position
		self.screen.blit(text,textpos)
		next_position = (next_position[0],next_position[1]+20)
		# .. Board Member Total
		font = pygame.font.Font(DEFAULT_FONT,12)
		text_str = "Board Members: "+str(self.player.get_total_boardmembers_killed())
		text = font.render(text_str,1,(192,192,192))
		textpos = next_position
		self.screen.blit(text,textpos)
		next_position = (next_position[0],next_position[1]+20)
		# .. Sycophant Total
		font = pygame.font.Font(DEFAULT_FONT,12)
		text_str = "Sycophants: "+str(self.player.get_total_sycophants_killed())
		text = font.render(text_str,1,(192,192,192))
		textpos = next_position
		self.screen.blit(text,textpos)
		next_position = (next_position[0],next_position[1]+20)
		# .. Player Deaths
		if self.player.own_deaths > 0:
			font = pygame.font.Font(DEFAULT_FONT,12)
			text_str = "Player Deaths: "+str(self.player.own_deaths)
			text = font.render(text_str,1,(192,192,192))
			textpos = next_position
			self.screen.blit(text,textpos)
			next_position = (next_position[0],next_position[1]+20)
				
	#/////////	
	# function
	# -- GET_RANDOM_NON_PLAYER_POSITION()
	#		- Returns a position in the window but not within the Player starting area
	#/////////
	def get_random_non_player_position(self):
		# .. Can't return a position in the Player "base", which is the space
		#			surrounding the Player Starting position
		position = (random.randrange(WORLD_LEFT,WORLD_RIGHT),random.randrange(WORLD_TOP,WORLD_BOTTOM))
		while self.playerbase_rect.collidepoint(position) or self.player.rect.collidepoint(position):
			position = (random.randrange(WORLD_LEFT,WORLD_RIGHT),random.randrange(WORLD_TOP,WORLD_BOTTOM))
		return(position)
		
	#/////////
	# function
	# -- ADD_MANAGER()
	#		- does the obvious and adds a Manager sprite
	#/////////
	def add_manager(self):
		manager = EnemyManagerSprite((random.randrange(WORLD_LEFT,WORLD_RIGHT),random.randrange(WORLD_TOP,WORLD_BOTTOM)))
		cycle_flag = 1
		while cycle_flag == 1:
			# .. so we get a Pos that doesn't collide with any other Managers
			while pygame.sprite.spritecollide(manager,self.manager_group,False):
				manager.kill()
				manager = EnemyManagerSprite((random.randrange(WORLD_LEFT,WORLD_RIGHT),random.randrange(WORLD_TOP,WORLD_BOTTOM)))
			# .. then we check if it collides with the Player
			if pygame.sprite.collide_rect(self.player,manager) == 0:
				cycle_flag = 0
			else:
				manager.kill()
				manager = EnemyManagerSprite((random.randrange(WORLD_LEFT,WORLD_RIGHT),random.randrange(WORLD_TOP,WORLD_BOTTOM)))				
		self.manager_group.add(manager)
		self.managers_alive += 1
		self.enemies_alive += 1

	def add_visitingsuits(self):
		visitingsuits = EnemyVisitingSuitsSprite((random.randrange(WORLD_LEFT,WORLD_RIGHT),random.randrange(WORLD_TOP,WORLD_BOTTOM)))
		cycle_flag = 1
		while cycle_flag == 1:
			# .. so we get a Pos that doesn't collide with any other VisitingSuits
			while pygame.sprite.spritecollide(visitingsuits,self.visitingsuits_group,False):
				visitingsuits.kill()
				visitingsuits = EnemyVisitingSuitsSprite((random.randrange(WORLD_LEFT,WORLD_RIGHT),random.randrange(WORLD_TOP,WORLD_BOTTOM)))
			# .. then we check against the Player
			if pygame.sprite.collide_rect(self.player,visitingsuits) == 0:
				cycle_flag = 0
			else:
				visitingsuits.kill()
				visitingsuits = EnemyVisitingSuitsSprite((random.randrange(WORLD_LEFT,WORLD_RIGHT),random.randrange(WORLD_TOP,WORLD_BOTTOM)))
		self.visitingsuits_group.add(visitingsuits)
		self.visitingsuits_alive += 1
		self.enemies_alive += 1
		
	def add_uppermanagement(self):
		uppermanagement = EnemyUpperManagementSprite((random.randrange(WORLD_LEFT,WORLD_RIGHT),random.randrange(WORLD_TOP,WORLD_BOTTOM)))
		cycle_flag = 1
		while cycle_flag == 1:
			# .. so we get a Pos that doesn't collide with any other Upper Management
			while pygame.sprite.spritecollide(uppermanagement,self.uppermanagement_group,False):
				uppermanagement.kill()
				uppermanagement = EnemyUpperManagementSprite((random.randrange(WORLD_LEFT,WORLD_RIGHT),random.randrange(WORLD_TOP,WORLD_BOTTOM)))
			# .. then we check against the Player
			if pygame.sprite.collide_rect(self.player,uppermanagement) == 0:
				cycle_flag = 0
			else:
				uppermanagement.kill()
				uppermanagement = EnemyUpperManagementSprite((random.randrange(WORLD_LEFT,WORLD_RIGHT),random.randrange(WORLD_TOP,WORLD_BOTTOM)))				
		self.uppermanagement_group.add(uppermanagement)
		self.uppermanagement_alive += 1	
		self.enemies_alive += 1
		
	def add_ceo(self):
		ceo = EnemyCEOSprite((random.randrange(WORLD_LEFT,WORLD_RIGHT),random.randrange(WORLD_TOP,WORLD_BOTTOM)))
		cycle_flag = 1
		while cycle_flag == 1:
			# .. so we get a Pos that doesn't collide with any other CEOs
			while pygame.sprite.spritecollide(ceo,self.ceo_group,False):
				ceo.kill()
				ceo = EnemyCEOSprite((random.randrange(WORLD_LEFT,WORLD_RIGHT),random.randrange(WORLD_TOP,WORLD_BOTTOM)))
			# .. then we check against the Player
			if pygame.sprite.collide_rect(self.player,ceo) == 0:
				cycle_flag = 0
			else:
				ceo.kill()
				ceo = EnemyCEOSprite((random.randrange(WORLD_LEFT,WORLD_RIGHT),random.randrange(WORLD_TOP,WORLD_BOTTOM)))				
		self.ceo_group.add(ceo)
		self.ceos_alive += 1	
		self.enemies_alive += 1	
		
	def add_boardmember(self):
		boardmember = EnemyBoardMemberSprite((random.randrange(WORLD_LEFT,WORLD_RIGHT),random.randrange(WORLD_TOP,WORLD_BOTTOM)))
		cycle_flag = 1
		while cycle_flag == 1:
			# .. so we get a Pos that doesn't collide with any other Board Members
			while pygame.sprite.spritecollide(boardmember,self.boardmember_group,False):
				boardmember.kill()
				boardmember = EnemyBoardMemberSprite((random.randrange(WORLD_LEFT,WORLD_RIGHT),random.randrange(WORLD_TOP,WORLD_BOTTOM)))
			# .. then we check against the Player
			if pygame.sprite.collide_rect(self.player,boardmember) == 0:
				cycle_flag = 0
			else:
				boardmember.kill()
				boardmember = EnemyBoardMemberSprite((random.randrange(WORLD_LEFT,WORLD_RIGHT),random.randrange(WORLD_TOP,WORLD_BOTTOM)))				
		self.boardmember_group.add(boardmember)
		self.boardmembers_alive += 1	
		self.enemies_alive += 1	
		
	def add_sycophant(self,position):
		sycophant = EnemySycophantSprite(position)
		self.sycophant_group.add(sycophant)
		self.sycophants_alive += 1
		self.enemies_alive += 1
		
	def add_boss(self):
		if self.boss_initialised == 0:
			self.boss_initialised = 1
			boss = EnemyBossSprite((random.randrange(WORLD_LEFT,WORLD_RIGHT),random.randrange(WORLD_TOP,WORLD_BOTTOM)))
			cycle_flag = 1
			while cycle_flag == 1:
				# .. so we get a Pos that doesn't collide with any other Board Members
				while pygame.sprite.spritecollide(boss,self.boss_group,False):
					boss.kill()
					boss = EnemyBossSprite((random.randrange(WORLD_LEFT,WORLD_RIGHT),random.randrange(WORLD_TOP,WORLD_BOTTOM)))
				# .. then we check against the Player
				if pygame.sprite.collide_rect(self.player,boss) == 0:
					cycle_flag = 0
				else:
					boss.kill()
					boss = EnemyBossSprite((random.randrange(WORLD_LEFT,WORLD_RIGHT),random.randrange(WORLD_TOP,WORLD_BOTTOM)))				
			self.boss_group.add(boss)
		
	def add_powerup_lifespan(self,position):
		powerup_lifespan = PowerupLifespanSprite(position)
		self.powerup_lifespan_group.add(powerup_lifespan)
		
	def add_powerup_damage(self,position):
		powerup_damage = PowerupDamageSprite(position)
		self.powerup_damage_group.add(powerup_damage)
		
	def add_powerup_multi(self,position):
		powerup_multi = PowerupMultiSprite(position)
		self.powerup_multi_group.add(powerup_multi)
		
	def add_powerup_through(self,position):
		powerup_through = PowerupThroughSprite(position)
		self.powerup_through_group.add(powerup_through)
	
	def add_powerup_health(self,position):
		powerup_health = PowerupHealthSprite(position)
		self.powerup_health_group.add(powerup_health)
		
	def add_powerup_weapon_two(self,position):
		powerup_weapon_two = PowerupWeaponTwoSprite(position)
		self.powerup_weapon_two_group.add(powerup_weapon_two)
		
	def add_powerup_maxhealth(self,position):
		powerup_maxhealth = PowerupMaxHealthSprite(position)
		self.powerup_maxhealth_group.add(powerup_maxhealth)
	
	def player_collide(self,deltat):
		#----
		# ENEMIES
		#----
		if self.player.state == PLAYERSTATE_ALIVE:
			# .. Managers
			collisions = pygame.sprite.spritecollide(self.player,self.manager_group,False)
			for s in collisions:
				if self.player.take_damage(MANAGER_DAMAGE,s) == 1:
					break
			if self.player.state == PLAYERSTATE_ALIVE:
				# .. Visiting Suits
				collisions = pygame.sprite.spritecollide(self.player,self.visitingsuits_group,False)
				for s in collisions:
					if self.player.take_damage(VISITINGSUITS_DAMAGE,s) == 1:
						break
				if self.player.state == PLAYERSTATE_ALIVE:
					# .. Upper Management
					collisions = pygame.sprite.spritecollide(self.player,self.uppermanagement_group,False)
					for s in collisions:
						if self.player.take_damage(UPPERMANAGEMENT_DAMAGE,s) == 1:
							break
					if self.player.state == PLAYERSTATE_ALIVE:
						# .. CEO
						collisions = pygame.sprite.spritecollide(self.player,self.ceo_group,False)
						for s in collisions:
							if self.player.take_damage(CEO_DAMAGE,s) == 1:
								break
						if self.player.state == PLAYERSTATE_ALIVE:
							# .. Board Member
							collisions = pygame.sprite.spritecollide(self.player,self.boardmember_group,False)
							for s in collisions:
								if self.player.take_damage(BOARDMEMBER_DAMAGE,s) == 1:
									break
							if self.player.state == PLAYERSTATE_ALIVE:
								# .. Sycophant
								collisions = pygame.sprite.spritecollide(self.player,self.sycophant_group,False)
								for s in collisions:
									if self.player.take_damage(SYCOPHANT_DAMAGE,s) == 1:
										break
								if self.player.state == PLAYERSTATE_ALIVE:
									# .. Sycophant Children
									collisions = pygame.sprite.spritecollide(self.player,self.sycophant_child_group,False)
									for s in collisions:
										if self.player.take_damage(SYCOPHANT_CHILD_DAMAGE,s) == 1:
											break
									# .. BOSS
									collisions = pygame.sprite.spritecollide(self.player,self.boss_group,False)
									for s in collisions:
										if self.player.take_damage(BOSS_DAMAGE,s) == 1:
											break
		#----
		# ENEMY WEAPONS
		#----
		if self.player.state == PLAYERSTATE_ALIVE:
			# .. Tasks
			collisions = pygame.sprite.spritecollide(self.player,self.weapon_task_group,False)
			for s in collisions:
				if self.player.take_damage(WEAPON_TASK_DAMAGE,s) == 1:
					break
				s.kill()
			if self.player.state == PLAYERSTATE_ALIVE:
				# .. Magic Wands
				collisions = pygame.sprite.spritecollide(self.player,self.weapon_magicwand_group,False)
				for s in collisions:
					if self.player.take_damage(WEAPON_MAGICWAND_DAMAGE,s) == 1:
						break
					s.kill()
				if self.player.state == PLAYERSTATE_ALIVE:
					# .. Credit Cards
					collisions = pygame.sprite.spritecollide(self.player,self.weapon_creditcard_group,False)
					for s in collisions:
						if self.player.take_damage(WEAPON_CREDITCARD_DAMAGE,s) == 1:
							break
						s.kill()
		
		#----
		# POWERUPS
		#----
		if self.player.state == PLAYERSTATE_ALIVE:
			# .. Lifespan
			collisions = pygame.sprite.spritecollide(self.player,self.powerup_lifespan_group,False)
			for s in collisions:
				self.player.current_weapon_powerup(POWERUP_LIFESPAN)
				s.pickup_sound.play()
				s.kill()
			# .. Damage
			collisions = pygame.sprite.spritecollide(self.player,self.powerup_damage_group,False)
			for s in collisions:
				self.player.current_weapon_powerup(POWERUP_DAMAGE)
				s.pickup_sound.play()
				s.kill()
			# .. Multi
			collisions = pygame.sprite.spritecollide(self.player,self.powerup_multi_group,False)
			for s in collisions:
				self.player.current_weapon_powerup(POWERUP_MULTI)
				s.pickup_sound.play()
				s.kill()
			# .. Through
			collisions = pygame.sprite.spritecollide(self.player,self.powerup_through_group,False)
			for s in collisions:
				self.player.current_weapon_powerup(POWERUP_THROUGH)
				s.pickup_sound.play()
				s.kill()
			# .. Health
			collisions = pygame.sprite.spritecollide(self.player,self.powerup_health_group,False)
			for s in collisions:
				self.player.current_weapon_powerup(POWERUP_HEALTH)
				s.pickup_sound.play()
				s.kill()
			# .. Weapon Two
			collisions = pygame.sprite.spritecollide(self.player,self.powerup_weapon_two_group,False)
			for s in collisions:
				self.player.weapon_two_powerup(POWERUP_COUNT)
				s.pickup_sound.play()
				s.kill()
			# .. Max Health
			collisions = pygame.sprite.spritecollide(self.player,self.powerup_maxhealth_group,False)
			for s in collisions:
				self.player.current_weapon_powerup(POWERUP_MAXHEALTH)
				s.pickup_sound.play()
				s.kill()
		
	def weapon_one_collide(self):
		# .. MANAGERS
		collisions = pygame.sprite.groupcollide(self.manager_group,self.weapon_one_group,False,False)
		for manager in collisions:
			# .. check Through logic
			for bullet in collisions[manager]:
				if manager <> bullet.last_enemy:
					bullet.through += 1
					if bullet.through > self.player.weapon_one_through:
						bullet.kill()
					# .. take weapon_one_damage away .. this resides in the Player, because might have been augmented	
					manager.current_health -= self.player.weapon_one_damage
				bullet.last_enemy = manager
			# .. Kill Manager
			if manager.current_health <= 0:
				self.managers_alive -= 1
				self.enemies_alive -= 1
				self.player.managers_killed += 1
				self.player.enemies_killed += 1
				print('Got one (Manager)!')
				# .. update Experience by Manager 
				self.player.increase_experience((SCORE_MANAGER * int(math.fabs(self.player.speed))))
				# .. Replication Logic
				self.replicate_enemies(ENEMY_MANAGER)
				# .. POWERUPS
				self.drop_powerups(manager.position,ENEMY_MANAGER)
				# .. Remove the Manager Object
				manager.death_sound.play()
				manager.kill()
				
		# .. VISITINGSUITS
		collisions = pygame.sprite.groupcollide(self.visitingsuits_group,self.weapon_one_group,False,False)
		for visitingsuits in collisions:
			# .. check Through logic
			for bullet in collisions[visitingsuits]:
				if visitingsuits <> bullet.last_enemy:
					bullet.through += 1
					if bullet.through > self.player.weapon_one_through:
						bullet.kill()
					# .. take weapon_one_damage away .. this resides in the Player, because might have been augmented	
					visitingsuits.current_health -= self.player.weapon_one_damage
				bullet.last_enemy = visitingsuits
			# .. Kill Visiting Suits
			if visitingsuits.current_health <= 0:
				self.visitingsuits_alive -= 1
				self.enemies_alive -= 1
				self.player.visitingsuits_killed += 1
				self.player.enemies_killed += 1
				print("Got one (Visiting Suits)!")
				# .. update Experience by VisitingSuits
				self.player.increase_experience((SCORE_VISITINGSUITS * int(math.fabs(self.player.speed))))
				# .. Replication Logic
				self.replicate_enemies(ENEMY_VISITINGSUITS)
				# .. POWERUPS
				self.drop_powerups(visitingsuits.position,ENEMY_VISITINGSUITS)
				# .. Remove the VisitingSuits Object
				visitingsuits.death_sound.play()
				visitingsuits.kill()	
				
		# .. UPPER MANAGEMENT
		collisions = pygame.sprite.groupcollide(self.uppermanagement_group,self.weapon_one_group,False,False)
		for uppermanagement in collisions:
			# .. check Through logic
			for bullet in collisions[uppermanagement]:
				if uppermanagement <> bullet.last_enemy:
					bullet.through += 1
					if bullet.through > self.player.weapon_one_through:
						bullet.kill()
					# .. take weapon_one_damage away .. this resides in the Player, because might have been augmented	
					uppermanagement.current_health -= self.player.weapon_one_damage
				bullet.last_enemy = uppermanagement
			# .. Kill Upper Management
			if uppermanagement.current_health <= 0:
				self.uppermanagement_alive -= 1
				self.enemies_alive -= 1
				self.player.uppermanagement_killed += 1
				self.player.enemies_killed += 1
				print("Got one (Upper Management)!")
				# .. update Experience by Upper Management
				self.player.increase_experience((SCORE_UPPERMANAGEMENT * int(math.fabs(self.player.speed))))				
				# .. Replication Logic
				self.replicate_enemies(ENEMY_UPPERMANAGEMENT)
				# .. POWERUPS
				self.drop_powerups(uppermanagement.position,ENEMY_UPPERMANAGEMENT)
				# .. Remove the Upper Management Object
				uppermanagement.death_sound.play()
				uppermanagement.kill()
				
		# .. CEO
		collisions = pygame.sprite.groupcollide(self.ceo_group,self.weapon_one_group,False,False)
		for ceo in collisions:
			# .. check Through logic
			for bullet in collisions[ceo]:
				if ceo <> bullet.last_enemy:
					bullet.through += 1
					if bullet.through > self.player.weapon_one_through:
						bullet.kill()
					# .. take weapon_one_damage away .. this resides in the Player, because might have been augmented	
					ceo.current_health -= self.player.weapon_one_damage
				bullet.last_enemy = ceo
			# .. Kill Upper Management
			if ceo.current_health <= 0:
				self.ceos_alive -= 1
				self.enemies_alive -= 1
				self.player.ceos_killed += 1
				self.player.enemies_killed += 1
				print("Got one (CEO)!")
				# .. update Experience by Upper Management
				self.player.increase_experience((SCORE_CEO * int(math.fabs(self.player.speed))))				
				# .. Replication Logic
				self.replicate_enemies(ENEMY_CEO)
				# .. POWERUPS
				self.drop_powerups(ceo.position,ENEMY_CEO)
				# .. Remove the Upper Management Object
				ceo.death_sound.play()
				ceo.kill()

		# .. BOARD MEMBER
		collisions = pygame.sprite.groupcollide(self.boardmember_group,self.weapon_one_group,False,False)
		for boardmember in collisions:
			# .. check Through logic
			for bullet in collisions[boardmember]:
				if boardmember <> bullet.last_enemy:
					bullet.through += 1
					if bullet.through > self.player.weapon_one_through:
						bullet.kill()
					# .. take weapon_one_damage away .. this resides in the Player, because might have been augmented	
					boardmember.current_health -= self.player.weapon_one_damage
				bullet.last_enemy = boardmember
			# .. Kill Board Member
			if boardmember.current_health <= 0:
				self.boardmembers_alive -= 1
				self.enemies_alive -= 1
				self.player.boardmembers_killed += 1
				self.player.enemies_killed += 1
				print("Got one (Board Member)!")
				# .. update Experience by Upper Management
				self.player.increase_experience((SCORE_BOARDMEMBER * int(math.fabs(self.player.speed))))				
				# .. Replication Logic
				self.replicate_enemies(ENEMY_BOARDMEMBER)
				# .. POWERUPS
				self.drop_powerups(boardmember.position,ENEMY_BOARDMEMBER)
				# .. Add Sycophant
				pos = (boardmember.position[0]+50,boardmember.position[1])
				self.add_sycophant(pos)
				pos = (boardmember.position[0]-50,boardmember.position[1])
				self.add_sycophant(pos)
				# .. Remove the Upper Management Object
				boardmember.death_sound.play()
				boardmember.kill()	
				
		# .. SYCOPHANT
		collisions = pygame.sprite.groupcollide(self.sycophant_group,self.weapon_one_group,False,False)
		for sycophant in collisions:
			# .. check Through logic
			for bullet in collisions[sycophant]:
				if sycophant <> bullet.last_enemy:
					bullet.through += 1
					if bullet.through > self.player.weapon_one_through:
						bullet.kill()
					# .. take weapon_one_damage away .. this resides in the Player, because might have been augmented	
					sycophant.current_health -= self.player.weapon_one_damage
				bullet.last_enemy = sycophant
			# .. Kill Sycophant
			if sycophant.current_health <= 0:
				self.sycophants_alive -= 1
				self.enemies_alive -= 1
				self.player.sycophants_killed += 1
				self.player.enemies_killed += 1
				print("Got one (Sycophant)!")
				# .. update Experience by Upper Management
				self.player.increase_experience((SCORE_SYCOPHANT * int(math.fabs(self.player.speed))))
				# .. there is no "Replication" logic for a Sycophant
				# .. POWERUPS
				self.drop_powerups(sycophant.position,ENEMY_SYCOPHANT)								
				# .. Remove the Sycophant Object
				sycophant.death_sound.play()
				sycophant.kill()

		# .. BOSS
		collisions = pygame.sprite.groupcollide(self.boss_group,self.weapon_one_group,False,False)
		for boss in collisions:
			# .. check Through logic
			for bullet in collisions[boss]:
				if boss <> bullet.last_enemy:
					bullet.through += 1
					if bullet.through > self.player.weapon_one_through:
						bullet.kill()
					# .. take weapon_one_damage away .. this resides in the Player, because might have been augmented	
					boss.current_health -= self.player.weapon_one_damage
				bullet.last_enemy = boss
			# .. Kill Boss
			if boss.current_health <= 0:
				print("Got the Boss! You have finished!")
				boss.death_sound.play()
				boss.kill()
				self.gamestate = STATE_FINISHED
	
	def weapon_two_collide(self):
		# .. MANAGERS
		collisions = pygame.sprite.groupcollide(self.manager_group,self.weapon_two_shrapnel_group,False,True)
		for manager in collisions:
			# .. take weapon_two_damage away .. this resides in the Player, because might have been augmented						
			manager.current_health -= self.player.weapon_two_damage
			if manager.current_health <= 0:						
				self.managers_alive -= 1
				self.enemies_alive -= 1
				self.player.managers_killed += 1
				self.player.enemies_killed += 1
				print('Got one (Manager)!')
				# .. update Experience by Manager 
				self.player.increase_experience((SCORE_MANAGER * int(math.fabs(self.player.speed))))
				# .. Replication Logic
				self.replicate_enemies(ENEMY_MANAGER)
				# .. POWERUPS
				self.drop_powerups(manager.position,ENEMY_MANAGER)
				# .. Remove the Manager Object
				manager.death_sound.play()
				manager.kill()	
		# .. VISITINGSUITS
		collisions = pygame.sprite.groupcollide(self.visitingsuits_group,self.weapon_two_shrapnel_group,False,True)
		for visitingsuits in collisions:
			# .. take weapon_two_damage away
			visitingsuits.current_health -= self.player.weapon_two_damage
			if visitingsuits.current_health <= 0:
				self.visitingsuits_alive -= 1
				self.enemies_alive -= 1
				self.player.visitingsuits_killed += 1
				self.player.enemies_killed += 1
				print("Got one (Visiting Suits)!")
				# .. update Experience by VisitingSuits
				self.player.increase_experience((SCORE_VISITINGSUITS * int(math.fabs(self.player.speed))))
				# .. Replication Logic
				self.replicate_enemies(ENEMY_VISITINGSUITS)
				# .. POWERUPS
				self.drop_powerups(visitingsuits.position,ENEMY_VISITINGSUITS)
				# .. Remove the Visiting Suits Object
				visitingsuits.death_sound.play()
				visitingsuits.kill()
		# .. UPPER MANAGEMENT
		collisions = pygame.sprite.groupcollide(self.uppermanagement_group,self.weapon_two_shrapnel_group,False,True)
		for uppermanagement in collisions:
			# .. take weapon_two_damage away
			uppermanagement.current_health -= self.player.weapon_two_damage
			if uppermanagement.current_health <= 0:
				self.uppermanagement_alive -= 1
				self.enemies_alive -= 1
				self.player.uppermanagement_killed += 1
				self.player.enemies_killed += 1
				print("Got one (Upper Management)!")
				# .. update Experience by uppermanagement
				self.player.increase_experience((SCORE_UPPERMANAGEMENT * int(math.fabs(self.player.speed))))
				# .. Replication Logic
				self.replicate_enemies(ENEMY_UPPERMANAGEMENT)
				# .. POWERUPS
				self.drop_powerups(uppermanagement.position,ENEMY_UPPERMANAGEMENT)
				# .. Remove the Upper Management Object
				uppermanagement.death_sound.play()
				uppermanagement.kill()
		# .. CEO
		collisions = pygame.sprite.groupcollide(self.ceo_group,self.weapon_two_shrapnel_group,False,True)
		for ceo in collisions:
			# .. take weapon_two_damage away
			ceo.current_health -= self.player.weapon_two_damage
			if ceo.current_health <= 0:
				self.ceos_alive -= 1
				self.enemies_alive -= 1
				self.player.ceos_killed += 1
				self.player.enemies_killed += 1
				print("Got one (Upper Management)!")
				# .. update Experience by CEO
				self.player.increase_experience((SCORE_CEO * int(math.fabs(self.player.speed))))
				# .. Replication Logic
				self.replicate_enemies(ENEMY_CEO)
				# .. POWERUPS
				self.drop_powerups(ceo.position,ENEMY_CEO)
				# .. Remove the CEO Object
				ceo.death_sound.play()
				ceo.kill()
		# .. BOARD MEMBER
		collisions = pygame.sprite.groupcollide(self.boardmember_group,self.weapon_two_shrapnel_group,False,True)
		for boardmember in collisions:
			# .. take weapon_two_damage away
			boardmember.current_health -= self.player.weapon_two_damage
			if boardmember.current_health <= 0:
				self.boardmembers_alive -= 1
				self.enemies_alive -= 1
				self.player.boardmembers_killed += 1
				self.player.enemies_killed += 1
				print("Got one (Upper Management)!")
				# .. update Experience by Board Member
				self.player.increase_experience((SCORE_BOARDMEMBER * int(math.fabs(self.player.speed))))
				# .. Replication Logic
				self.replicate_enemies(ENEMY_BOARDMEMBER)
				# .. POWERUPS
				self.drop_powerups(boardmember.position,ENEMY_BOARDMEMBER)
				# .. Add Sycophant
				self.add_sycophant(boardmember.position)
				self.add_sycophant(boardmember.position)
				self.add_sycophant(boardmember.position)
				# .. Remove the Visiting Suits Object
				boardmember.death_sound.play()
				boardmember.kill()
		# .. SYCOPHANT
		collisions = pygame.sprite.groupcollide(self.sycophant_group,self.weapon_two_shrapnel_group,False,True)
		for sycophant in collisions:
			# .. take weapon_two_damage away
			sycophant.current_health -= self.player.weapon_two_damage
			if sycophant.current_health <= 0:
				self.sycophants_alive -= 1
				self.enemies_alive -= 1
				self.player.sycophants_killed += 1
				self.player.enemies_killed += 1
				print("Got one (Upper Management)!")
				# .. update Experience by SYCOPHANT
				self.player.increase_experience((SCORE_SYCOPHANT * int(math.fabs(self.player.speed))))
				# .. there is no "Replication" logic for a Sycophant
				# .. POWERUPS
				self.drop_powerups(sycophant.position,ENEMY_SYCOPHANT)				
				# .. Remove the Sycophant Object
				sycophant.death_sound.play()
				sycophant.kill()
		# .. BOSS
		collisions = pygame.sprite.groupcollide(self.boss_group,self.weapon_two_shrapnel_group,False,True)
		for boss in collisions:
			boss.current_health -= self.player.weapon_two_damage
			# .. Kill Boss
			if boss.current_health <= 0:
				print("Got the Boss! You have finished!")
				boss.death_sound.play()
				boss.kill()
				self.gamestate = STATE_FINISHED
		
	def weapon_three_collide(self):
		# .. MANAGERS
		collisions = pygame.sprite.groupcollide(self.manager_group,self.weapon_three_group,False,False)
		for manager in collisions:
			# .. check Through logic
			for bullet in collisions[manager]:
				if manager <> bullet.last_enemy:
					bullet.through += 1
					if bullet.through > self.player.weapon_three_through:
						bullet.kill()
					# .. take weapon_three_damage away .. this resides in the Player, because might have been augmented	
					manager.current_health -= self.player.weapon_three_damage
				bullet.last_enemy = manager
			# .. Kill Manager
			if manager.current_health <= 0:
				self.managers_alive -= 1
				self.enemies_alive -= 1
				self.player.managers_killed += 1
				self.player.enemies_killed += 1
				print('Got one (Manager)!')
				# .. update Experience by Manager 
				self.player.increase_experience((SCORE_MANAGER * int(math.fabs(self.player.speed))))
				# .. Replication Logic
				self.replicate_enemies(ENEMY_MANAGER)
				# .. POWERUPS
				self.drop_powerups(manager.position,ENEMY_MANAGER)
				# .. Remove the Manager Object
				manager.death_sound.play()
				manager.kill()
				
		# .. VISITINGSUITS
		collisions = pygame.sprite.groupcollide(self.visitingsuits_group,self.weapon_three_group,False,False)
		for visitingsuits in collisions:
			# .. check Through logic
			for bullet in collisions[visitingsuits]:
				if visitingsuits <> bullet.last_enemy:
					bullet.through += 1
					if bullet.through > self.player.weapon_three_through:
						bullet.kill()
					# .. take weapon_three_damage away .. this resides in the Player, because might have been augmented	
					visitingsuits.current_health -= self.player.weapon_three_damage
				bullet.last_enemy = visitingsuits
			# .. Kill Visiting Suits
			if visitingsuits.current_health <= 0:
				self.visitingsuits_alive -= 1
				self.enemies_alive -= 1
				self.player.visitingsuits_killed += 1
				self.player.enemies_killed += 1
				print("Got one (Visiting Suits)!")
				# .. update Experience by VisitingSuits
				self.player.increase_experience((SCORE_VISITINGSUITS * int(math.fabs(self.player.speed))))
				# .. Replication Logic
				self.replicate_enemies(ENEMY_VISITINGSUITS)
				# .. POWERUPS
				self.drop_powerups(visitingsuits.position,ENEMY_VISITINGSUITS)
				# .. Remove the VisitingSuits Object
				visitingsuits.death_sound.play()
				visitingsuits.kill()	
				
		# .. UPPER MANAGEMENT
		collisions = pygame.sprite.groupcollide(self.uppermanagement_group,self.weapon_three_group,False,False)
		for uppermanagement in collisions:
			# .. check Through logic
			for bullet in collisions[uppermanagement]:
				if uppermanagement <> bullet.last_enemy:
					bullet.through += 1
					if bullet.through > self.player.weapon_three_through:
						bullet.kill()
					# .. take weapon_three_damage away .. this resides in the Player, because might have been augmented	
					uppermanagement.current_health -= self.player.weapon_three_damage
				bullet.last_enemy = uppermanagement
			# .. Kill Upper Management
			if uppermanagement.current_health <= 0:
				self.uppermanagement_alive -= 1
				self.enemies_alive -= 1
				self.player.uppermanagement_killed += 1
				self.player.enemies_killed += 1
				print("Got one (Upper Management)!")
				# .. update Experience by Upper Management
				self.player.increase_experience((SCORE_UPPERMANAGEMENT * int(math.fabs(self.player.speed))))				
				# .. Replication Logic
				self.replicate_enemies(ENEMY_UPPERMANAGEMENT)
				# .. POWERUPS
				self.drop_powerups(uppermanagement.position,ENEMY_UPPERMANAGEMENT)
				# .. Remove the Upper Management Object
				uppermanagement.death_sound.play()
				uppermanagement.kill()
				
		# .. CEO
		collisions = pygame.sprite.groupcollide(self.ceo_group,self.weapon_three_group,False,False)
		for ceo in collisions:
			# .. check Through logic
			for bullet in collisions[ceo]:
				if ceo <> bullet.last_enemy:
					bullet.through += 1
					if bullet.through > self.player.weapon_three_through:
						bullet.kill()
					# .. take weapon_three_damage away .. this resides in the Player, because might have been augmented	
					ceo.current_health -= self.player.weapon_three_damage
				bullet.last_enemy = ceo
			# .. Kill Upper Management
			if ceo.current_health <= 0:
				self.ceos_alive -= 1
				self.enemies_alive -= 1
				self.player.ceos_killed += 1
				self.player.enemies_killed += 1
				print("Got one (CEO)!")
				# .. update Experience by Upper Management
				self.player.increase_experience((SCORE_CEO * int(math.fabs(self.player.speed))))				
				# .. Replication Logic
				self.replicate_enemies(ENEMY_CEO)
				# .. POWERUPS
				self.drop_powerups(ceo.position,ENEMY_CEO)
				# .. Remove the Upper Management Object
				ceo.death_sound.play()
				ceo.kill()

		# .. BOARD MEMBER
		collisions = pygame.sprite.groupcollide(self.boardmember_group,self.weapon_three_group,False,False)
		for boardmember in collisions:
			# .. check Through logic
			for bullet in collisions[boardmember]:
				if boardmember <> bullet.last_enemy:
					bullet.through += 1
					if bullet.through > self.player.weapon_three_through:
						bullet.kill()
					# .. take weapon_three_damage away .. this resides in the Player, because might have been augmented	
					boardmember.current_health -= self.player.weapon_three_damage
				bullet.last_enemy = boardmember
			# .. Kill Board Member
			if boardmember.current_health <= 0:
				self.boardmembers_alive -= 1
				self.enemies_alive -= 1
				self.player.boardmembers_killed += 1
				self.player.enemies_killed += 1
				print("Got one (Board Member)!")
				# .. update Experience by Upper Management
				self.player.increase_experience((SCORE_BOARDMEMBER * int(math.fabs(self.player.speed))))				
				# .. Replication Logic
				self.replicate_enemies(ENEMY_BOARDMEMBER)
				# .. POWERUPS
				self.drop_powerups(boardmember.position,ENEMY_BOARDMEMBER)
				# .. Add Sycophant
				pos = (boardmember.position[0]+50,boardmember.position[1])
				self.add_sycophant(pos)
				pos = (boardmember.position[0]-50,boardmember.position[1])
				self.add_sycophant(pos)
				# .. Remove the Upper Management Object
				boardmember.death_sound.play()
				boardmember.kill()	
				
		# .. SYCOPHANT
		collisions = pygame.sprite.groupcollide(self.sycophant_group,self.weapon_three_group,False,False)
		for sycophant in collisions:
			# .. check Through logic
			for bullet in collisions[sycophant]:
				if sycophant <> bullet.last_enemy:
					bullet.through += 1
					if bullet.through > self.player.weapon_three_through:
						bullet.kill()
					# .. take weapon_three_damage away .. this resides in the Player, because might have been augmented	
					sycophant.current_health -= self.player.weapon_three_damage
				bullet.last_enemy = sycophant
			# .. Kill Sycophant
			if sycophant.current_health <= 0:
				self.sycophants_alive -= 1
				self.enemies_alive -= 1
				self.player.sycophants_killed += 1
				self.player.enemies_killed += 1
				print("Got one (Sycophant)!")
				# .. update Experience by Upper Management
				self.player.increase_experience((SCORE_SYCOPHANT * int(math.fabs(self.player.speed))))
				# .. there is no "Replication" logic for a Sycophant
				# .. POWERUPS
				self.drop_powerups(sycophant.position,ENEMY_SYCOPHANT)								
				# .. Remove the Sycophant Object
				sycophant.death_sound.play()
				sycophant.kill()	
				
		# .. BOSS
		collisions = pygame.sprite.groupcollide(self.boss_group,self.weapon_three_group,False,False)
		for boss in collisions:
			# .. check Through logic
			for bullet in collisions[boss]:
				if boss <> bullet.last_enemy:
					bullet.through += 1
					if bullet.through > self.player.weapon_three_through:
						bullet.kill()
					# .. take weapon_one_damage away .. this resides in the Player, because might have been augmented	
					boss.current_health -= self.player.weapon_three_damage
				bullet.last_enemy = boss
			# .. Kill Boss
			if boss.current_health <= 0:
				print("Got the Boss! You have finished!")
				boss.death_sound.play()
				boss.kill()
				self.gamestate = STATE_FINISHED
				self.player.state = PLAYERSTATE_DEAD
	
	def drop_powerups(self,position,parent_type):
		# .. Init
		powerup_dropped = 0
		# .. Set SIMPLE adjustment 
		adjust = 0
		if parent_type == ENEMY_MANAGER:
			adjust = 0
		elif parent_type == ENEMY_VISITINGSUITS:
			adjust = 5
		elif parent_type == ENEMY_UPPERMANAGEMENT:
			adjust = 50
		elif parent_type == ENEMY_CEO:
			adjust = 75
		elif parent_type == ENEMY_BOARDMEMBER:
			adjust = 95
		elif parent_type == ENEMY_SYCOPHANT:
			adjust = 50
		adjust += self.player.speed
		adjust += self.player.level
		# .. Which Powerup?
		choice = random.randint(1,7)
		# Lifespan
		if choice == 1:
			if random.randint(1,100) > 85-adjust:
				self.add_powerup_lifespan(position)
				powerup_dropped = 1
		# Damage
		elif choice == 2:
			if random.randint(1,100) > 90-adjust:
				self.add_powerup_damage(position)
				powerup_dropped = 1
		# Multi
		elif choice == 3:
			if random.randint(1,100) > 97-adjust:
				self.add_powerup_multi(position)
				powerup_dropped = 1
		# Through
		elif choice == 4:
			if random.randint(1,100) > 90-adjust:
				self.add_powerup_through(position)
				powerup_dropped = 1
		# Health
		elif choice == 5:
			if random.randint(1,100) > 90-adjust:
				self.add_powerup_health(position)
				powerup_dropped = 1
		# Weapon Two
		elif choice == 6:
			if (random.randint(1,100) > 50-adjust) and (self.player.level >= WEAPON_TWO_LEVEL):
				self.add_powerup_weapon_two(position)
				powerup_dropped = 1
		# Max Health
		elif choice == 7:
			if (random.randint(1,100) > 95-adjust):
				self.add_powerup_maxhealth(position)
				powerup_dropped = 1
		#----
		# POWERUP NOT DROPPED
		# .. Okay, now we want to check and see if something dropped .. if not, go through and check for dropping Health (and maybe more)
		#----
		if powerup_dropped == 0:
			# .. Health
			if (random.randint(1,100) > 75-adjust):
				self.add_powerup_health(position)
				powerup_dropped = 1
		
	def replicate_enemies(self,parent_type):
		#----
		# MANAGER_LEVEL
		#----
		if self.player.level >= MANAGER_LEVEL:
			# .. REPLICATE MANAGER logic, Replicate once, and then percentage chance of replicate again
			if self.managers_alive > 0:
				if self.enemies_alive < MAX_ENEMIES_ALIVE:
					if random.randint(1,100) > 55:
						self.add_manager()
						if random.randint(1,100) > 90:
							self.add_manager()
			else:
				# .. always have one on the screen
				self.add_manager()
				if random.randint(1,100) > 55:
						self.add_manager()
			#----
			# VISITINGSUITS_LEVEL
			#----
			if self.player.level >= VISITINGSUITS_LEVEL:	
				# .. REPLICATE VISITINGSUITS logic, Replicate once, and then percentage chance of replicate again
				if self.visitingsuits_alive > 0:
					if self.enemies_alive < MAX_ENEMIES_ALIVE:
						if random.randint(1,100) > 75:
							self.add_visitingsuits()
							if random.randint(1,100) > 95:
								self.add_visitingsuits()
				else:
					self.add_visitingsuits()
					if random.randint(1,100) > 85:
							self.add_visitingsuits()
				#----
				# UPPERMANAGEMENT_LEVEL
				#----
				if self.player.level >= UPPERMANAGEMENT_LEVEL:
					# .. REPLICATE UPPER MANAGEMENT logic, Replicate once, and then percentage chance of replicate again
					if self.uppermanagement_alive > 0:
						if self.enemies_alive < MAX_ENEMIES_ALIVE:
							if random.randint(1,100) > 85:
								self.add_uppermanagement()
								if random.randint(1,100) > 99:
									self.add_uppermanagement()
					else:
						self.add_uppermanagement()
						if random.randint(1,100) > 95:
								self.add_uppermanagement()
					#----
					# CEO_LEVEL
					#----
					if self.player.level >= CEO_LEVEL:
						# .. REPLICATE CEO logic, Replicate once, and then percentage chance of replicate again
						if self.ceos_alive > 0:
							if self.enemies_alive < MAX_ENEMIES_ALIVE:
								if random.randint(1,100) > 93:
									self.add_ceo()
									if random.randint(1,100) > 99:
										self.add_ceo()
						else:
							self.add_ceo()
							if random.randint(1,100) > 95:
									self.add_ceo()
						#----
						# BOARDMEMBER_LEVEL
						#----
						if self.player.level >= BOARDMEMBER_LEVEL:
							# .. REPLICATE Board Member logic, Replicate once, and then percentage chance of replicate again
							if self.boardmembers_alive > 0:
								if self.enemies_alive < MAX_ENEMIES_ALIVE:
									if random.randint(1,100) > 97:
										self.add_boardmember()
										if random.randint(1,100) > 99:
											self.add_boardmember()
							else:
								self.add_boardmember()
								if random.randint(1,100) > 99:
										self.add_boardmember()
							#----
							# BOSS_LEVEL
							#----
							if self.player.level >= BOSS_LEVEL:
								# .. REPLICATE Boss logic, Replicate once, and then percentage chance of replicate again
								self.add_boss()

	def clear_player_base(self):
		# .. Managers
		collisions = pygame.sprite.spritecollide(self.player,self.manager_group,False)
		for s in collisions:
			s.kill()
			self.managers_alive -= 1
			self.add_manager()
		# .. Visiting Suits
		collisions = pygame.sprite.spritecollide(self.player,self.visitingsuits_group,False)
		for s in collisions:
			s.kill()
			self.visitingsuits_alive -= 1
			self.add_visitingsuits()
		# .. Upper Management
		collisions = pygame.sprite.spritecollide(self.player,self.uppermanagement_group,False)
		for s in collisions:
			s.kill()
			self.uppermanagement_alive -= 1
			self.add_uppermanagement()
		# .. CEO
		collisions = pygame.sprite.spritecollide(self.player,self.ceo_group,False)
		for s in collisions:
			s.kill()
			self.ceos_alive -= 1
			self.add_ceo()
		# .. Board Members
		collisions = pygame.sprite.spritecollide(self.player,self.boardmember_group,False)
		for s in collisions:
			s.kill()
			self.boardmembers_alive -= 1
			self.add_boardmember()
		# .. Sycophants
		collisions = pygame.sprite.spritecollide(self.player,self.sycophant_group,False)
		for s in collisions:
			s.kill()
			self.sycophants_alive -= 1
			# .. don't add another one
		# .. Sycophant Children
		collisions = pygame.sprite.spritecollide(self.player,self.sycophant_child_group,False)
		for s in collisions:
			s.kill()
			
	def set_icon(self):
		fullname = os.path.join('data','images')
		fullname = os.path.join(fullname,'spaceofficefight-32.png')
		icon = pygame.Surface((32,32))
		icon.set_colorkey((0,0,0))
		raw_icon = pygame.image.load(fullname)
		for i in range(0,32):
			for j in range(0,32):
				icon.set_at((i,j), raw_icon.get_at((i,j)))
		pygame.display.set_icon(icon)
		
	def get_closest_enemy(self,origin_point):
		closest_point = vec2d(0,0)
		closest_distance = 0
		closest_direction = 0
		closest_speed = 0
		clear = 1
		# .. Check Managers
		for s in self.manager_group:
			new_point = vec2d(s.position[0],s.position[1])
			new_distance = new_point.get_distance(origin_point)
			if closest_distance == 0 or new_distance < closest_distance:
				closest_distance = new_distance
				closest_point = new_point
				closest_direction = s.direction
				closest_speed = s.speed
		# .. Check VisitingSuits
		for s in self.visitingsuits_group:
			new_point = vec2d(s.position[0],s.position[1])
			new_distance = new_point.get_distance(origin_point)
			if closest_distance == 0 or new_distance < closest_distance:
				closest_distance = new_distance
				closest_point = new_point
				closest_direction = s.direction
				closest_speed = s.speed
		# .. Check Upper Management
		for s in self.uppermanagement_group:
			new_point = vec2d(s.position[0],s.position[1])
			new_distance = new_point.get_distance(origin_point)
			if closest_distance == 0 or new_distance < closest_distance:
				closest_distance = new_distance
				closest_point = new_point
				closest_direction = s.direction
				closest_speed = s.speed
		# .. Check CEO
		for s in self.ceo_group:
			new_point = vec2d(s.position[0],s.position[1])
			new_distance = new_point.get_distance(origin_point)
			if closest_distance == 0 or new_distance < closest_distance:
				closest_distance = new_distance
				closest_point = new_point
				closest_direction = s.direction
				closest_speed = s.speed				
		# .. Check Board Member
		for s in self.boardmember_group:
			new_point = vec2d(s.position[0],s.position[1])
			new_distance = new_point.get_distance(origin_point)
			if closest_distance == 0 or new_distance < closest_distance:
				closest_distance = new_distance
				closest_point = new_point
				closest_direction = s.direction
				closest_speed = s.speed
		# .. Check Sycophant
		for s in self.sycophant_group:
			new_point = vec2d(s.position[0],s.position[1])
			new_distance = new_point.get_distance(origin_point)
			if closest_distance == 0 or new_distance < closest_distance:
				closest_distance = new_distance
				closest_point = new_point
				closest_direction = s.direction
				closest_speed = s.speed

		return closest_point,closest_direction, closest_speed
	
	def new_game(self):
		self.show_menu = 0
		self.gamestate = STATE_PLAY
		self.player.state = PLAYERSTATE_ALIVE
		rect = self.screen.get_rect()
		self.player.rebirth(rect.center)
		self.player.clear_stats()
		self.enemies_alive = 0
		# .. Clear Managers
		self.manager_group.empty()
		self.init_managers()
		self.managers_alive = 0
		# .. Clear Visiting Suits
		self.visitingsuits_group.empty()
		self.init_visitingsuits()
		self.visitingsuits_alive = 0
		# .. Clear Upper Management
		self.uppermanagement_group.empty()
		self.init_uppermanagement()
		self.uppermanagement_alive = 0
		# .. Clear CEO
		self.ceo_group.empty()
		self.init_ceos()
		self.ceos_alive = 0
		# .. Clear Board Member
		self.boardmember_group.empty()
		self.init_boardmembers()
		self.boardmembers_alive = 0
		# .. Clear Sycophant
		self.sycophant_group.empty()
		self.init_sycophants()
		self.sycophantss_alive = 0
		# .. Clear Sycophant Children
		self.sycophant_child_group.empty()
		self.init_sycophant_children()
		# .. Clear Weapons
		self.weapon_one_group.empty()
		self.weapon_two_group.empty()
		self.weapon_two_shrapnel_group.empty()
		self.weapon_three_base_group.empty()			# not really sure if this should go here, it's NOT a weapon
		self.weapon_three_group.empty()
		self.weapon_task_group.empty()
		self.weapon_magicwand_group.empty()
		self.weapon_creditcard_group.empty()
		# .. Clear Powerups
		self.powerup_lifespan_group.empty()
		self.powerup_damage_group.empty()
		self.powerup_multi_group.empty()
		self.powerup_through_group.empty()
		self.powerup_health_group.empty()
		self.powerup_weapon_two_group.empty()
	
#====
# Call Main 
# .. EXTREMELY important to have this, otherwise nothing will run
#====
if __name__ == "__main__":
	MainWindow = GameMain()
	MainWindow.MainLoop()