#====
# Player Class
#====

import os, sys, math
import pygame
from pygame.locals import *
from equates import *
from helpers import *
from weapon_one import *
from weapon_two import *
from weapon_three import *
from weapon_three_base import *

class PlayerSprite(pygame.sprite.Sprite):
	MAX_FORWARD_SPEED = 10
	MAX_REVERSE_SPEED = -5
	ACCELERATION = 2
	TURN_SPEED = 5

	def __init__(self,image,position):
		pygame.sprite.Sprite.__init__(self)
		self.image, self.rect = load_image(image,None,position)
		if self.rect == None:
			self.rect = self.image.get_rect()			
		# .. Base	
		self.src_image = self.image
		self.position = position
		#self.rect.center = self.position
		self.speed = self.direction = 0
		self.k_left = self.k_right = self.k_down = self.k_up = 0
		self.current_weapon = WEAPON_ONE
		self.just_levelled = 0
		# .. General 
		self.enemies_killed = self.managers_killed = self.visitingsuits_killed = self.uppermanagement_killed = self.ceos_killed = self.boardmembers_killed = self.sycophants_killed = 0
		self.state = PLAYERSTATE_ALIVE
		self.max_health = PLAYER_START_HEALTH
		self.current_health = self.max_health
		self.own_deaths = 0
		self.health_loss_sound = load_sound('player_health_loss.ogg')
		self.health_loss_sound.set_volume(0.5)
		self.death_sound = load_sound('player_death.ogg')
		self.death_sound.set_volume(0.75)
		# .. Weapon One
		self.weapon_one_sound = load_sound('weapon_one.ogg')
		self.weapon_one_sound.set_volume(0.5)
		self.weapon_one_damage = WEAPON_ONE_DAMAGE
		self.weapon_one_lifespan = WEAPON_LIFESPAN_SHORT
		self.weapon_one_multi = WEAPON_ONE_MULTI		# how many "bullets" are shot in one go
		self.weapon_one_through = WEAPON_ONE_THROUGH
		# .. Weapon Two
		self.weapon_two_count = 1
		self.weapon_two_damage = WEAPON_TWO_DAMAGE
		self.weapon_two_lifespan = WEAPON_LIFESPAN_LONG
		self.weapon_two_multi = WEAPON_TWO_MULTI
		self.weapon_two_through = WEAPON_TWO_THROUGH
		self.weapon_two_empty_sound = load_sound('weapon_two_empty.ogg')
		self.weapon_two_empty_sound.set_volume(0.75)
		# .. Weapon Three
		self.weapon_three_damage = WEAPON_THREE_DAMAGE
		self.weapon_three_lifespan = WEAPON_LIFESPAN_SHORT
		self.weapon_three_multi = WEAPON_THREE_MULTI
		self.weapon_three_through = WEAPON_THREE_THROUGH
		self.weapon_three_initialised = 0
		# .. Level/Experience
		self.level = self.experience = 0
		self.level_up_sound = load_sound('player_level_up.ogg')
		# .. Damage
		self.previous_damage_sprite = 0
		self.previous_damage_timer = 0
		
	def update(self,deltat,weapon_three_base_group,weapon_three_group):
		# .. Init
		self.previous_damage_timer += deltat
		# .. Change Direction	
		self.speed += (self.k_up + self.k_down)
		if self.speed > self.MAX_FORWARD_SPEED:
			self.speed = self.MAX_FORWARD_SPEED
		if self.speed < self.MAX_REVERSE_SPEED:
			self.speed = self.MAX_REVERSE_SPEED
		self.direction += (self.k_right + self.k_left)
		
		if self.direction > 359:
			self.direction -= 360
		if self.direction < 0:
			self.direction += 360
		# .. Calc New Position
		x,y = self.position
		rad = self.direction * math.pi / 180
		x += -self.speed*math.sin(rad)
		y += -self.speed*math.cos(rad)
		# .. boundary check
		if x < 0: x = MAX_XPOS
		elif x > MAX_XPOS: x = 0
		if y < 0: y = MAX_YPOS
		elif y > MAX_YPOS: y = 0
		# .. continue
		self.position = (x,y)
		self.image = pygame.transform.rotate(self.src_image,self.direction)
		self.rect = self.image.get_rect()
		self.rect.center = self.position
		# .. Weapon Three Base
		if self.level >= WEAPON_THREE_LEVEL:
			# .. get position/direction 
			base_position = get_next_point(self.position,get_opposite_direction(self.direction),WEAPON_THREE_BASE_DISTANCE)
			base_direction = self.direction
			# .. add?
			if self.weapon_three_initialised == 0:
				weapon_three_base = WeaponThreeBaseSprite(base_position)
				weapon_three_base_group.add(weapon_three_base)
				self.weapon_three_initialised = 1
			# .. update()
			weapon_three_base_group.update(deltat,base_position,base_direction,weapon_three_group,self.weapon_three_multi)
	
	def get_total_managers_killed(self):
		return self.managers_killed

	def get_total_visitingsuits_killed(self):
		return self.visitingsuits_killed
		
	def get_total_uppermanagement_killed(self):
		return self.uppermanagement_killed
		
	def get_total_ceos_killed(self):
		return self.ceos_killed
		
	def get_total_boardmembers_killed(self):
		return self.boardmembers_killed
		
	def get_total_sycophants_killed(self):
		return self.sycophants_killed
		
	def rebirth(self,position):
		# .. Go for it
		self.position = position
		self.speed = 0		# left out self.direction
		self.k_left = self.k_right = self.k_down = self.k_up = 0
		rect = self.image.get_rect()
		xpos = position[0] - rect.width / 2
		ypos = position[1] - rect.height / 2
		self.rect = pygame.Rect(xpos,ypos,rect.width,rect.height)
		self.current_health = self.max_health
		
	def clear_stats(self):
		# .. General
		self.experience = self.managers_killed = self.visitingsuits_killed = self.uppermanagement_killed = self.ceos_killed = self.boardmembers_killed = self.sycophants_killed = 0
		self.level = 0
		self.current_health = self.max_health
		self.current_weapon = WEAPON_ONE
		# .. Weapon One
		self.weapon_one_damage = WEAPON_ONE_DAMAGE
		self.weapon_one_lifespan = WEAPON_LIFESPAN_SHORT
		self.weapon_one_multi = WEAPON_ONE_MULTI
		self.weapon_one_through = WEAPON_ONE_THROUGH
		# .. Weapon Two
		self.weapon_two_damage = WEAPON_TWO_DAMAGE
		self.weapon_two_lifespan = WEAPON_LIFESPAN_VERYSHORT
		self.weapon_two_multi = WEAPON_TWO_MULTI
		self.weapon_two_through = WEAPON_TWO_THROUGH
		# .. Weapon Three
		self.weapon_three_damage = WEAPON_THREE_DAMAGE
		self.weapon_three_lifespan = WEAPON_LIFESPAN_MEDIUM
		self.weapon_three_multi = WEAPON_THREE_MULTI
		self.weapon_three_through = WEAPON_THREE_THROUGH
		
	def increase_experience(self,experience_gained):
		self.just_levelled = 0
		self.experience += experience_gained
		if self.experience < 0: self.experience = 0
		while self.level < LEVELS_MAX:
			if self.level == 0:
				if self.experience >= LEVEL_ONE_EXP:
					self.level = 1
					self.just_levelled = 1
					if self.experience < LEVEL_TWO_EXP:
						break
				else:
					break
			elif self.level == 1:
				if self.experience >= LEVEL_TWO_EXP:
					self.level = 2
					self.just_levelled = 1
					if self.experience < LEVEL_THREE_EXP:
						break
				else:
					if self.experience < LEVEL_ONE_EXP:
						self.level = 0
					else:
						break
			elif self.level == 2:
				if self.experience >= LEVEL_THREE_EXP:
					self.level = 3
					self.just_levelled = 1
					if self.experience < LEVEL_FOUR_EXP:
						break
				else:
					if self.experience < LEVEL_TWO_EXP:
						self.level = 1
					else:
						break
			elif self.level == 3:
				if self.experience >= LEVEL_FOUR_EXP:
					self.level = 4
					self.just_levelled = 1
					if self.experience < LEVEL_FIVE_EXP:
						break
				else:
					if self.experience < LEVEL_THREE_EXP:
						self.level = 2
					else:
						break
			elif self.level == 4:
				if self.experience >= LEVEL_FIVE_EXP:
					self.level = 5
					self.just_levelled = 1
					if self.experience < LEVEL_SIX_EXP:
						break
				else:
					if self.experience < LEVEL_FOUR_EXP:
						self.level = 3
					else:
						break
			elif self.level == 5:
				if self.experience >= LEVEL_SIX_EXP:
					self.level = 6
					self.just_levelled = 1
					if self.experience < LEVEL_SEVEN_EXP:
						break
				else:
					if self.experience < LEVEL_FIVE_EXP:
						self.level = 4
					else:
						break
			elif self.level == 6:
				if self.experience >= LEVEL_SEVEN_EXP:
					self.level = 7
					self.just_levelled = 1
					if self.experience < LEVEL_EIGHT_EXP:
						break
				else:
					if self.experience < LEVEL_SIX_EXP:
						self.level = 5
					else:
						break
			elif self.level == 7:
				if self.experience >= LEVEL_EIGHT_EXP:
					self.level = 8
					self.just_levelled = 1
					if self.experience < LEVEL_NINE_EXP:
						break
				else:
					if self.experience < LEVEL_SEVEN_EXP:
						self.level = 6
					else:
						break
			elif self.level == 8:
				if self.experience >= LEVEL_NINE_EXP:
					self.level = 9
					self.just_levelled = 1
					if self.experience < LEVEL_TEN_EXP:
						break
				else:
					if self.experience < LEVEL_EIGHT_EXP:
						self.level = 7
					else:
						break
			elif self.level == 9:
				if self.experience >= LEVEL_TEN_EXP:
					self.level = 10
					self.just_levelled = 1
					break
				else:
					if self.experience < LEVEL_NINE_EXP:
						self.level = 8
					else:
						break
			elif self.level == 10:
				if self.experience < LEVEL_TEN_EXP:
					self.level = 9
		# .. Just Levelled?
		if self.just_levelled == 1:
			self.current_health = self.max_health
			self.level_up_sound.play()
		
	def get_next_level_experience(self):
		if self.level == 0:
			return LEVEL_ONE_EXP
		elif self.level == 1:
			return LEVEL_TWO_EXP
		elif self.level == 2:
			return LEVEL_THREE_EXP
		elif self.level == 3:
			return LEVEL_FOUR_EXP
		elif self.level == 4:
			return LEVEL_FIVE_EXP
		elif self.level == 5:
			return LEVEL_SIX_EXP
		elif self.level == 6:
			return LEVEL_SEVEN_EXP
		elif self.level == 7:
			return LEVEL_EIGHT_EXP
		elif self.level == 8:
			return LEVEL_NINE_EXP
		elif self.level == 9:
			return LEVEL_TEN_EXP
		elif self.level == 10:
			return "EOF .. Bzzzzz"
	
	def get_player_death_exp(self):
		if self.level == 0:
			return LEVEL_ONE_EXP / 10
		elif self.level == 1:
			return LEVEL_TWO_EXP / 10
		elif self.level == 2:
			return LEVEL_THREE_EXP / 10
		elif self.level == 3:
			return LEVEL_FOUR_EXP / 10
		elif self.level == 4:
			return LEVEL_FIVE_EXP / 10
		elif self.level == 5:
			return LEVEL_SIX_EXP / 10
		elif self.level == 6:
			return LEVEL_SEVEN_EXP / 10
		elif self.level == 7:
			return LEVEL_EIGHT_EXP / 10
		elif self.level == 8:
			return LEVEL_NINE_EXP / 10
		elif self.level == 9:
			return LEVEL_TEN_EXP / 10
		elif self.level == 10:
			return 100
			
	def current_weapon_powerup(self,powerup_type):
		# .. Lifespan
		if powerup_type == POWERUP_LIFESPAN:
			if (self.current_weapon == WEAPON_ONE) and (self.weapon_one_lifespan < WEAPON_MAX_LIFESPAN):
				self.weapon_one_lifespan += POWERUP_LIFESPAN_VALUE
			elif (self.current_weapon == WEAPON_TWO) and (self.weapon_two_lifespan < WEAPON_MAX_LIFESPAN):
				self.weapon_two_lifespan += POWERUP_LIFESPAN_VALUE
			# .. Weapon Three, always update 
			self.weapon_three_lifespan += POWERUP_LIFESPAN_VALUE
		# .. Damage
		elif powerup_type == POWERUP_DAMAGE:
			if (self.current_weapon == WEAPON_ONE) and (self.weapon_one_damage < WEAPON_MAX_DAMAGE):
				self.weapon_one_damage += POWERUP_DAMAGE_VALUE
			elif (self.current_weapon == WEAPON_TWO) and (self.weapon_two_damage < WEAPON_MAX_DAMAGE):
				self.weapon_two_damage += POWERUP_DAMAGE_VALUE
			# .. Weapon Three?
			self.weapon_three_damage += POWERUP_DAMAGE_VALUE
		# .. Multi
		elif powerup_type == POWERUP_MULTI:
			if (self.current_weapon == WEAPON_ONE) and (self.weapon_one_multi < WEAPON_MAX_MULTI):
				self.weapon_one_multi += POWERUP_MULTI_VALUE		# will almost certainly be 1
			elif (self.current_weapon == WEAPON_TWO) and (self.weapon_two_multi < WEAPON_MAX_MULTI):
				self.weapon_two_multi += POWERUP_MULTI_VALUE
			# .. Weapon Three?
			self.weapon_three_multi += POWERUP_MULTI_VALUE
		# .. Through
		elif powerup_type == POWERUP_THROUGH:
			if (self.current_weapon == WEAPON_ONE) and (self.weapon_one_through < WEAPON_MAX_THROUGH):
				self.weapon_one_through += POWERUP_THROUGH_VALUE
			elif (self.current_weapon == WEAPON_TWO) and (self.weapon_two_multi < WEAPON_MAX_THROUGH):
				self.weapon_two_through += POWERUP_THROUGH_VALUE
			# .. Weapon Three?
			self.weapon_three_through += POWERUP_THROUGH_VALUE
		# .. Health
		elif powerup_type == POWERUP_HEALTH:
			self.current_health = self.max_health
		# .. Max Health
		elif powerup_type == POWERUP_MAXHEALTH:
			self.max_health += POWERUP_HEALTH_VALUE
			self.current_health = self.max_health		

	def weapon_two_powerup(self,powerup_type):
		# .. Count
		if powerup_type == POWERUP_COUNT:
			self.weapon_two_count += 1
			
	def fire_weapon(self,weapon_type,weapon_sprite_group):
		# .. Weapon One
		if weapon_type == WEAPON_ONE:
			self.weapon_one_sound.play()		# do this first
			angle_delta = 0
			for i in range(self.weapon_one_multi):
				weapon_sprite_group.add(WeaponOneSprite(self.position,self.direction+angle_delta,self.weapon_one_lifespan))
				# .. update Experience by Weapon
				self.increase_experience(SCORE_WEAPON_ONE*(self.level+1))
				if angle_delta >= 0:
					angle_delta = -(i+1*5)
				else:
					angle_delta = i+1*5
		# .. Weapon Two
		elif weapon_type == WEAPON_TWO:
			# .. Check Level
			if self.level >= WEAPON_TWO_LEVEL:
				# .. Check Count
				if self.weapon_two_count > 0:
					angle_delta = 0
					for i in range(self.weapon_two_multi):
						weapon_sprite_group.add(WeaponTwoSprite(self.position,self.direction+angle_delta,self.weapon_two_lifespan))
						# .. update Experience by Weapon
						self.increase_experience(SCORE_WEAPON_TWO*(self.level+1))
						if angle_delta >= 0:
							angle_delta = -(i+1*5)
						else:
							angle_delta = i+1*5
						self.weapon_two_count -= 1
						if self.weapon_two_count == 0:
							break
				else:
					# .. No Bombs left
					self.weapon_two_empty_sound.play()
	
	def draw_weapon_details(self,display,weapon_type,topleft_position):
		if self.level >= WEAPON_TWO_LEVEL:
			#//////
			# BOMB
			#/////
			next_position = (topleft_position[0]-100,topleft_position[1])
			lifespan_value = self.weapon_two_lifespan
			damage_value = self.weapon_two_damage		
			# .. Heading
			font = pygame.font.Font(DEFAULT_FONT,12)
			text_str = "Bomb"
			text = font.render(text_str,1,(255,255,255))
			textpos = next_position
			display.blit(text,textpos)
			next_position = (next_position[0],next_position[1]+20)
			# .. Count
			font = pygame.font.Font(DEFAULT_FONT,12)
			text_str = "Count: "+str(self.weapon_two_count)
			text = font.render(text_str,1,(192,192,192))
			textpos = next_position
			display.blit(text,textpos)
			next_position = (next_position[0],next_position[1]+20)	
			# .. Lifespan
			font = pygame.font.Font(DEFAULT_FONT,12)
			text_str = "Lifespan: "+str(lifespan_value)
			text = font.render(text_str,1,(192,192,192))
			textpos = next_position
			display.blit(text,textpos)
			next_position = (next_position[0],next_position[1]+20)
			# .. Damage
			font = pygame.font.Font(DEFAULT_FONT,12)
			text_str = "Damage: "+str(damage_value)
			text = font.render(text_str,1,(192,192,192))
			textpos = next_position
			display.blit(text,textpos)
			next_position = (next_position[0],next_position[1]+20)		
		#//////
		# MAIN WEAPON
		#//////
		next_position = topleft_position
		# .. Heading
		font = pygame.font.Font(DEFAULT_FONT,12)
		text_str = "Main Weapon"
		text = font.render(text_str,1,(255,255,255))
		textpos = next_position
		display.blit(text,textpos)
		next_position = (next_position[0],next_position[1]+20)
		# .. Weapon One
		if weapon_type == WEAPON_ONE:
			lifespan_value = self.weapon_one_lifespan
			damage_value = self.weapon_one_damage
			multi_value = self.weapon_one_multi
			through_value = self.weapon_one_through			
		# .. Lifespan
		font = pygame.font.Font(DEFAULT_FONT,12)
		text_str = "Lifespan: "+str(lifespan_value)
		text = font.render(text_str,1,(192,192,192))
		textpos = next_position
		display.blit(text,textpos)
		next_position = (next_position[0],next_position[1]+20)
		# .. Damage
		font = pygame.font.Font(DEFAULT_FONT,12)
		text_str = "Damage: "+str(damage_value)
		text = font.render(text_str,1,(192,192,192))
		textpos = next_position
		display.blit(text,textpos)
		next_position = (next_position[0],next_position[1]+20)
		# .. Multi
		font = pygame.font.Font(DEFAULT_FONT,12)
		text_str = "Multi: "+str(multi_value)
		text = font.render(text_str,1,(192,192,192))
		textpos = next_position
		display.blit(text,textpos)
		next_position = (next_position[0],next_position[1]+20)
		# .. Through
		font = pygame.font.Font(DEFAULT_FONT,12)
		text_str = "Through: "+str(through_value)
		text = font.render(text_str,1,(192,192,192))
		textpos = next_position
		display.blit(text,textpos)
		next_position = (next_position[0],next_position[1]+20)		
		
	def draw_player_details(self,display,topleft_position):
		next_position = topleft_position
		# .. Heading
		font = pygame.font.Font(DEFAULT_FONT,12)
		text_str = "Player Details"
		text = font.render(text_str,1,(255,255,255))
		textpos = next_position
		display.blit(text,textpos)
		next_position = (next_position[0],next_position[1]+20)
		# .. Experience
		font = pygame.font.Font(DEFAULT_FONT,12)
		text_str = "Experience: "+str(self.experience)+"/"+str(self.get_next_level_experience())
		text = font.render(text_str,1,(192,192,192))
		textpos = next_position
		display.blit(text,textpos)
		next_position = (next_position[0],next_position[1]+20)
		# .. Level
		font = pygame.font.Font(DEFAULT_FONT,12)
		text_str = "Level: "+str(self.level)
		text = font.render(text_str,1,(192,192,192))
		textpos = next_position
		display.blit(text,textpos)
		next_position = (next_position[0],next_position[1]+20)
		#----
		# Health
		# .. little more complicated, want to color and resize the text based on Health Percentage
		#----
		percentage = float(self.current_health) / float(self.max_health) * 100
		#print("percentage: "+str(percentage)+", current health: "+str(self.current_health)+", max health: "+str(self.max_health))
		color = (153,0,0) #(192,192,192)
		if percentage > 0 and percentage <= 10:
			color = (153,0,0)
		elif percentage > 10 and percentage <= 25:
			color = (153,51,0)
		elif percentage > 25 and percentage <= 50:
			color = (153,102,0)
		elif percentage > 50 and percentage <= 75:
			color = (153,153,0)
		elif percentage > 75 and percentage <= 90:
			color = (153,204,0)
		elif percentage > 90:
			color = (153,255,0)
		font = pygame.font.Font(DEFAULT_FONT,12)
		text_str = "Health: "+str(self.current_health)+"/"+str(self.max_health)
		text = font.render(text_str,1,color)
		textpos = next_position
		display.blit(text,textpos)
		next_position = (next_position[0],next_position[1]+20)
		
		return textpos[0] + font.size(text_str)[0] + 50
	
	def take_damage(self,damage_amt,the_sprite):
		if (the_sprite <> self.previous_damage_sprite) or (self.previous_damage_timer > 550):			# self.previous_damage_timer is incremented in the update() proc
			self.previous_damage_sprite = the_sprite
			self.previous_damage_timer = 0
			self.current_health -= damage_amt		# would be better if we got the damage from the Enemy
			if self.current_health <= 0:
				self.state = PLAYERSTATE_DEAD
				self.own_deaths += 1
				print("You were TASKED!")
				# .. update Experience by Player Death
				self.increase_experience(SCORE_PLAYER_DEATH*(self.level+1))
				self.death_sound.play()
				return 1
			else:
				self.health_loss_sound.play()
			return 0

	def killed(self):
		self.state = PLAYERSTATE_DEAD
		self.own_deaths += 1
		print("You have been FILED!")
		# .. update Experience by Player Death
		self.increase_experience(SCORE_PLAYER_DEATH*(self.level+1))
		self.death_sound.play()
		
	def clear_powerups(self):
		# .. Weapon One
		self.weapon_one_damage = WEAPON_ONE_DAMAGE
		self.weapon_one_lifespan = WEAPON_LIFESPAN_SHORT
		self.weapon_one_multi = WEAPON_ONE_MULTI		# how many "bullets" are shot in one go
		self.weapon_one_through = WEAPON_ONE_THROUGH
		# .. Weapon Two
		#self.weapon_two_count = 1				# want to leave the bombs (got to do with the same reason leave max health alone)
		self.weapon_two_damage = WEAPON_TWO_DAMAGE
		self.weapon_two_lifespan = WEAPON_LIFESPAN_LONG
		self.weapon_two_multi = WEAPON_TWO_MULTI
		self.weapon_two_through = WEAPON_TWO_THROUGH
